@@ -32,6 +32,11 @@
 #include "config.h"
 #endif
 
+#if defined _WIN32 || defined WIN32 || defined __WIN32__ || defined __CYGWIN__ || defined __CYGWIN32__ || defined __MSVCRT__
+# define WIN32_LEAN_AND_MEAN
+# include <windows.h>
+#endif
+
 #ifdef STDC_HEADERS
 # include <string.h>
 # include <stdlib.h>
@@ -68,9 +73,9 @@ getpagesize(void)
   return NBPG * CLSIZE;
 #else	/* no NBPG */
 #ifdef	NBPC
-  return NBPC
+  return NBPC;
 #else	/* no NBPC */
-  return 4096;			/* Just punt and use reasonable value */
+  return 1024 * sizeof (long);		/* Just punt and use reasonable value */
 #endif /* no NBPC */
 #endif /* no NBPG */
 #endif /* no EXEC_PAGESIZE */
