@@ -34,19 +34,8 @@
  *
  ***********************************************************************/
 
-
-#include "gst.h"
 #include "gstpriv.h"
 
-#include <stdio.h>
-#if STDC_HEADERS
-#include <stdlib.h>
-#endif /* STDC_HEADERS */
-
-#include <stdarg.h>
-
-#include <math.h>
-
 #ifndef NAN
 #define NAN (0.0 / 0.0)
 #endif
