@@ -30,43 +30,10 @@
  ***********************************************************************/
 
 
-#include "gst.h"
 #include "gstpriv.h"
 
 #include <ltdl.h>
 
-#ifdef STDC_HEADERS
-# include <string.h>
-# include <stdlib.h>
-#endif /* STDC_HEADERS */
-
-#include <sys/types.h>
-#include <stdio.h>
-#include <errno.h>
-#include <sys/stat.h>
-
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
-#ifdef HAVE_DIRENT_H
-# include <dirent.h>
-# define NAMLEN(dirent) strlen((dirent)->d_name)
-#else
-# define dirent direct
-# define NAMLEN(dirent) (dirent)->d_namlen
-# ifdef HAVE_SYS_NDIR_H
-#  include <sys/ndir.h>
-# endif
-# ifdef HAVE_SYS_DIR_H
-#  include <sys/dir.h>
-# endif
-# ifdef HAVE_NDIR_H
-#  include <ndir.h>
-# endif
-#endif
-
-
 #define ARG_VEC_SIZE		20	/* 20 shorts, 10 longs or ptrs, 5 dbls */
 
 typedef enum
@@ -239,9 +206,6 @@ static char *extract_dirent_name (struct dirent *dir);
 static int get_argc (void);
 static char *get_argv (int n);
 
-/* The errno on output from a callout */
-static int saved_errno;
-
 /* The binary tree of function names vs. function addresses. */
 static cfunc_info *c_func_root = NULL;
 
@@ -334,6 +298,10 @@ static const int type_sizes[] = {
   sizeof (float),		/* CDATA_FLOAT */
   sizeof (double)		/* CDATA_DOUBLE */
 };
+
+/* The errno on output from a callout */
+int _gst_errno;
+
 
 
 
@@ -349,10 +317,10 @@ marli (int n)
 int
 get_errno (void)
 {
-  int returned;
-  returned = saved_errno;
-  saved_errno = 0;
-  return (returned);
+  int old;
+  old = _gst_errno;
+  _gst_errno = 0;
+  return (old);
 }
 
 int
@@ -1062,16 +1030,7 @@ call_cfunction (gst_cfunc_descriptor desc)
       break;
     }
 
-  /* ENOTEMPTY and EEXIST are synonymous; some systems use one, and
-     some use the other. We always uses EEXIST which is provided by all 
-     systems. */
-
-#ifdef ENOTEMPTY
-  saved_errno = (errno == ENOTEMPTY) ? EEXIST : errno;
-#else
-  saved_errno = errno;
-#endif
-
+  _gst_set_errno (errno);
   return resultOOP;
 }
 
@@ -1190,6 +1149,20 @@ classify_type_symbol (OOP symbolOOP,
   return (FROM_INT (CDATA_UNKNOWN));
 }
 
+void
+_gst_set_errno(int errnum)
+{
+  /* ENOTEMPTY and EEXIST are synonymous; some systems use one, and
+     some use the other. We always uses EEXIST which is provided by all 
+     systems. */
+
+#ifdef ENOTEMPTY
+  _gst_errno = (errnum == ENOTEMPTY) ? EEXIST : errnum;
+#else
+  _gst_errno = errnum;
+#endif
+}
+
 void
 _gst_restore_cfunc_descriptor (OOP cFuncDescOOP)
 {
