@@ -29,30 +29,7 @@
  *
  ***********************************************************************/
 
-
-#include "gst.h"
 #include "gstpriv.h"
-#include "obstack.h"
-
-#include <setjmp.h>
-#include <assert.h>
-#include <stdio.h>		/* for printf */
-
-#if defined(HAVE_SYS_TIME_H) || defined(TIME_WITH_SYS_TIME)
-# include <sys/time.h>
-#endif
-#if !defined(HAVE_SYS_TIME_H) || defined(TIME_WITH_SYS_TIME)
-# include <time.h>
-#endif
-
-#ifdef HAVE_SYS_RESOURCE_H
-# include <sys/resource.h>
-#endif
-
-#if STDC_HEADERS
-#include <string.h>
-#include <stdlib.h>
-#endif /* STDC_HEADERS */
 
 /* Define this if you want declaration tracing to print the bytecodes
    both *before* and *after* the optimizer is ran.  Default behavior
