@@ -29,17 +29,8 @@
  ***********************************************************************/
 
 
-#include "gst.h"
 #include "gstpriv.h"
 
-#include <stdio.h>
-#include <float.h>
-#include <fcntl.h>
-#if STDC_HEADERS
-#include <string.h>
-#include <stdlib.h>
-#endif /* STDC_HEADERS */
-
 /* this must be big enough that the Smalltalk dictionary does not have to
    grow between the time gst_dictionary is loaded and the time the kernel is
    initialized.  Otherwise some of the methods needed to grow the dictionary
@@ -1273,7 +1264,7 @@ _gst_find_class_method (OOP class_oop,
     (gst_identity_dictionary) OOP_TO_OBJ (method_dictionary_oop);
 
 #ifndef OPTIMIZE
-  if (!IS_OOP_ADDR (ARRAY_AT (methodDictionary->values, index + 1))
+  if (!IS_OOP_ADDR (ARRAY_AT (methodDictionary->values, index + 1)))
     abort ();
 #endif
 
