@@ -1424,8 +1424,8 @@ gen_proto (const char *s)
 {
   filprintf (proto_fil,
 	     "static long\n"
-	     "%s (int id,\n"
-	     "%*svolatile int numArgs);\n\n",
+	     "%s (int id ATTRIBUTE_UNUSED,\n"
+	     "%*svolatile int numArgs ATTRIBUTE_UNUSED);\n\n",
 	     s, 2 + strlen(s), "");
 }
 
