@@ -35,15 +35,14 @@
 #include "config.h"
 #include "gst.h"
 
+#if SIZEOF_CHAR_P != SIZEOF_LONG
+#error longs and pointer have different sizes
+#endif
+
+#if SIZEOF_INT < 4
+#error 16-bits platform are not supported.
+#endif
 
-/* ENABLE_SECURITY enables security checks in the primitives as well as
-   special marking of untrusted objects.  Note that the code in the
-   class library to perform the security checks will be present
-   notwithstanding the setting of this flag, but they will be disabled
-   because the corresponding primitives will be made non-working.
-   It is undefined because the Makefiles take care of defining it for
-   security-enabled builds.  */
-#define ENABLE_SECURITY
 
 /* OPTIMIZE disables many checks, including consistency checks at GC
    time and bounds checking on instance variable accesses (not on #at:
@@ -52,7 +51,7 @@
    debugging painful because you know of a bug only when it's too
    late.  It is undefined because the Makefiles take care of defining
    it for optimized builds.  Bounds-checking and other errors will
-   call abort().  */
+   call abort(). */
 /* #define OPTIMIZE */
 
 typedef unsigned char gst_uchar;
@@ -67,7 +66,7 @@ typedef unsigned char gst_uchar;
 #  endif
 #endif
 
-/* If they have no const, they're likely to have no volatile, either.  */
+/* If they have no const, they're likely to have no volatile, either. */
 #ifdef const
 #define volatile
 #endif
@@ -77,16 +76,29 @@ extern char *strdup ();
 /* else it is in string.h */
 #endif
 
+/* Implementation note: on 64 bit architectures, bit-encoded 32 bit
+   words are made to live in the low order 32 bits -- the high 32 bits
+   are ignored, and are padded if necessary to achieve the useful
+   stuff in the lower 32.  This simplifies the current implementation,
+   as most bit fields, both in C and in Smalltalk do not have to be
+   modified when on a 64 bit architecture.  Smalltalk Integers will be
+   able to represent up to 62 bits + 1 sign bit. */
+#if SIZEOF_LONG == 8
+#define LONG_SHIFT 3		/* bits to shift for a long or pointer */
+#else
+#define LONG_SHIFT 2		/* bits to shift for a long or pointer */
+#endif
+
+/* Fallback to BSD functions if the standard ones are not available. */
+#ifndef HAVE_MEMCPY
+#define memcpy(s1, s2, n) bcopy((s2), (s1), (n))
+#define memcmp(s1, s2, n) bcmp ((s1), (s2), (n))
+#endif /* HAVE_MEMCPY */
+
 
 /* Run-time flags are allocated from the top, while those
    that live across image saves/restores are allocated
-   from the bottom.
-
-   bit 0-3: reserved for distinguishing byte objects and saving their size.
-   bit 4-11: non-volatile bits (special kinds of objects).
-   bit 12-23: volatile bits (GC/JIT-related).
-   bit 24-30: reserved for counting things.
-   bit 31: unused to avoid signedness mess. */
+   from the bottom. */
 enum {
   /* Place to save various OOP counts (how many fields we have marked
      in the object, how many pointer instance variables there are, 
@@ -102,29 +114,29 @@ enum {
   F_COUNT_SHIFT = 24,
 
   /* Set if the object is reachable, during the mark phases of oldspace
-     garbage collection.  */
+     garbage collection. */
   F_REACHABLE = 0x800000U,
 
   /* Set if a translation to native code is available, when running
-     with the JIT compiler enabled.  */
+     with the JIT compiler enabled. */
   F_XLAT = 0x400000U,
 
   /* Set if a translation to native code is used by the currently
-     reachable contexts.  */
+     reachable contexts. */
   F_XLAT_REACHABLE = 0x200000U,
 
   /* Set if a translation to native code is available but not used by
      the reachable contexts at the time of the last GC.  We give
      another chance to the object, but if the translation is not used
-     for two consecutive GCs we discard it.  */
+     for two consecutive GCs we discard it. */
   F_XLAT_2NDCHANCE = 0x100000U,
 
   /* Set if a translation to native code was discarded for this
      object (either because the programmer asked for this, or because
-     the method conflicted with a newly-installed method).  */
+     the method conflicted with a newly-installed method. */
   F_XLAT_DISCARDED = 0x80000U,
 
-  /* One of this is set for objects that live in newspace.  */
+  /* One of this is set for objects that live in newspace. */
   F_SPACES = 0x60000U,
   F_EVEN = 0x40000U,
   F_ODD = 0x20000U,
@@ -133,89 +145,64 @@ enum {
      in interp.c (maybe it belongs above...) */
   F_POOLED = 0x10000U,
 
-  /* Set if the bytecodes in the method have been verified. */
-  F_VERIFIED = 0x8000U,
-
   /* The grouping of all the flags which are not valid across image
-     saves and loads.  */
-  F_RUNTIME = 0xFFF000U,
+     saves and loads. */
+  F_RUNTIME = 0xFF0000U,
 
-  /* Set if the OOP is currently unused.  */
-  F_FREE = 0x10U,
+  /* Set if the OOP is currently unused. */
+  F_FREE = 0x100U,
 
   /* Set if the references to the instance variables of the object
-     are weak.  */
-  F_WEAK = 0x20U,
+     are weak. */
+  F_WEAK = 0x200U,
 
-  /* Set if the object is read-only.  */
-  F_READONLY = 0x40U,
+  /* Set if the object is read-only. */
+  F_READONLY = 0x400U,
 
   /* Set if the object is a context and hence its instance variables
-     are only valid up to the stack pointer.  */
-  F_CONTEXT = 0x80U,
+     are only valid up to the stack pointer. */
+  F_CONTEXT = 0x800U,
 
   /* Answer whether we want to mark the key based on references found
      outside the object.  */
-  F_EPHEMERON = 0x100U,
+  F_EPHEMERON = 0x1000U,
 
-  /* Set for objects that live in oldspace.  */
-  F_OLD = 0x200U,
+  /* Set for objects that live in oldspace. */
+  F_OLD = 0x2000U,
 
-  /* Set together with F_OLD for objects that live in fixedspace.  */
-  F_FIXED = 0x400U,
-
-  /* Set for untrusted classes, instances of untrusted classes,
-     and contexts whose receiver is untrusted.  */
-  F_UNTRUSTED = 0x800U,
+  /* Set together with F_OLD for objects that live in fixedspace. */
+  F_FIXED = 0x4000U,
 
   /* Set to the number of bytes unused in an object with byte-sized
      instance variables.  Note that this field and the following one
      should be initialized only by INIT_BYTE_OBJECT (not really 
      aesthetic but...) */
-  EMPTY_BYTES = (sizeof (PTR) - 1),
+  EMPTY_BYTES = (SIZEOF_LONG - 1),
 
   /* A bit more than what is identified by EMPTY_BYTES.  Selects some
      bits that are never zero if and only if this OOP identifies an
      object with byte instance variables.  */
-  F_BYTE = EMPTY_BYTES | sizeof (PTR)
+  F_BYTE = EMPTY_BYTES | SIZEOF_LONG
 };
 
-/* Answer whether a method, OOP, has already been verified. */
-#define IS_OOP_VERIFIED(oop) \
-  ((oop)->flags & F_VERIFIED)
-
-/* Answer whether an object, OOP, is weak.  */
+/* Answer whether an object, OOP, is readonly. */
 #define IS_OOP_WEAK(oop) \
-  ((oop)->flags & F_WEAK)
+  (((oop)->flags & F_WEAK) != 0)
 
-/* Answer whether an object, OOP, is readonly.  */
+/* Answer whether an object, OOP, is readonly. */
 #define IS_OOP_READONLY(oop) \
-  (IS_INT ((oop)) ? F_READONLY : (oop)->flags & F_READONLY)
+  (((oop)->flags & F_READONLY) != 0)
 
-/* Set whether an object, OOP, is readonly or readwrite.  */
-#define MAKE_OOP_READONLY(oop, ro) \
-  (((oop)->flags &= ~F_READONLY), \
-   ((oop)->flags |= (ro) ? F_READONLY : 0))
+/* Set whether an object, OOP, is readonly or readwrite. */
+#define MAKE_OOP_READONLY(oop, ro) do { \
+  if (ro) { (oop)->flags |= F_READONLY; } \
+  else { (oop)->flags &= ~F_READONLY; } \
+} while(0)
 
-#ifdef ENABLE_SECURITY
-
-/* Answer whether an object, OOP, is untrusted.  */
-#define IS_OOP_UNTRUSTED(oop) \
-  (IS_INT ((oop)) ? 0 : ((oop)->flags & F_UNTRUSTED))
-
-/* Set whether an object, OOP, is trusted or untrusted.  */
-#define MAKE_OOP_UNTRUSTED(oop, untr) \
-  (((oop)->flags &= ~F_UNTRUSTED), \
-   ((oop)->flags |= (untr) ? F_UNTRUSTED : 0))
-
-#else
-#define IS_OOP_UNTRUSTED(oop) 0
-#define MAKE_OOP_UNTRUSTED(oop, untr) 0
-#endif
-
-/* Set whether an object, OOP, has ephemeron semantics.  */
-#define MAKE_OOP_EPHEMERON(oop) \
-  (oop)->flags |= F_EPHEMERON;
+/* Set whether an object, OOP, has a finalizer. */
+#define MAKE_OOP_EPHEMERON(oop) do { \
+  (oop)->flags |= F_EPHEMERON; \
+} while(0)
 
 
 /*
@@ -231,33 +218,33 @@ enum {
    Remember to shift by ISP_NUMFIXEDFIELDS-1 there, since Smalltalk does
    not see ISP_INTMARK!!
 
-   FIXME: these should be exported in a pool dictionary.  */
+   FIXME: these should be exported in a pool dictionary. */
 
 enum {
-  /* This is a shift count.  */
+  /* This is a shift count. */
   ISP_NUMFIXEDFIELDS = 13,
 
   /* These represent the shape of the indexed instance variables of
-     the instances of the class.  */
+     the instances of the class. */
   ISP_INDEXEDVARS = 28,         
 
-  /* Set if the indexed instance variables are pointers.  */
+  /* Set if the indexed instance variables are pointers. */
   ISP_ISPOINTERS = 16,
 
-  /* Set if the indexed instance variables are words (C longs).  */
+  /* Set if the indexed instance variables are words (C longs). */
   ISP_ISWORDS = 8,
 
   /* Set if the instances of the class have indexed instance
-     variables.  */
+     variables. */
   ISP_ISINDEXABLE = 4,
 
-  /* Set to 1 to mark a SmallInteger.  */
+  /* Set to 1 to mark a SmallInteger. */
   ISP_INTMARK = 1
 };
 
 
 /* the current execution stack pointer */
-#ifndef ENABLE_JIT_TRANSLATION
+#ifndef USE_JIT_TRANSLATION
 # define sp		_gst_sp
 #endif
 
@@ -277,17 +264,12 @@ extern OOP _gst_nil_oop, _gst_true_oop, _gst_false_oop;
 #ifndef OPTIMIZE
 #define PUSH_OOP(oop) \
   do { \
-    OOP __pushOOP = (oop); \
-    if (IS_OOP (__pushOOP) && !IS_OOP_VALID (__pushOOP)) \
-      abort (); \
-    UNCHECKED_PUSH_OOP (__pushOOP); \
+    if (!IS_OOP_COPIED (oop)) abort (); \
+    UNCHECKED_PUSH_OOP (oop); \
   } while (0)
 #else
 #define PUSH_OOP(oop) \
-  do { \
-    OOP __pushOOP = (oop); \
-    UNCHECKED_PUSH_OOP (__pushOOP); \
-  } while (0)
+    UNCHECKED_PUSH_OOP(oop)
 #endif
 
 #define POP_OOP() \
@@ -305,17 +287,12 @@ extern OOP _gst_nil_oop, _gst_true_oop, _gst_false_oop;
 #ifndef OPTIMIZE
 #define SET_STACKTOP(oop) \
   do { \
-    OOP __pushOOP = (oop); \
-    if (IS_OOP (__pushOOP) && !IS_OOP_VALID (__pushOOP)) \
-      abort (); \
-    UNCHECKED_SET_TOP(__pushOOP); \
+    if (!IS_OOP_COPIED (oop)) abort (); \
+    UNCHECKED_SET_TOP(oop); \
   } while (0)
 #else
 #define SET_STACKTOP(oop) \
-  do { \
-    OOP __pushOOP = (oop); \
-    UNCHECKED_SET_TOP(__pushOOP); \
-  } while (0)
+  UNCHECKED_SET_TOP(oop)
 #endif
 
 #define SET_STACKTOP_INT(i) \
@@ -334,21 +311,15 @@ extern OOP _gst_nil_oop, _gst_true_oop, _gst_false_oop;
   TO_INT(POP_OOP())
 
 #define PUSH_BOOLEAN(exp) \
-  PUSH_OOP((exp) ? _gst_true_oop : _gst_false_oop)
+  UNCHECKED_PUSH_OOP((exp) ? _gst_true_oop : _gst_false_oop)
 
 
 /* Answer whether CLASS is the class that the object pointed to by OOP
-   belongs to.  OOP can also be a SmallInteger.  */
+   belongs to.  OOP can also be a SmallInteger. */
 #define IS_CLASS(oop, class) \
-  (OOP_INT_CLASS(oop) == class)
+  (IS_INT(oop) ? class == _gst_small_integer_class : class == OOP_CLASS(oop))
 
-/* Answer the CLASS that the object pointed to by OOP belongs to.  OOP
-   can also be a SmallInteger. */
-#define OOP_INT_CLASS(oop) \
-  (IS_INT(oop) ? _gst_small_integer_class : OOP_CLASS(oop))
-
-
-/* Answer whether OOP is nil.  */
+/* Answer whether OOP is nil. */
 #define IS_NIL(oop) \
   ((OOP)(oop) == _gst_nil_oop)
 
@@ -357,65 +328,65 @@ extern OOP _gst_nil_oop, _gst_true_oop, _gst_false_oop;
    emptyBytes field is guaranteed to be zero.
 
    Note that F_BYTE is a bit more than EMPTY_BYTES, so that if value 
-   is a multiple of sizeof (PTR) the flags identified by F_BYTE are
+   is a multiple of SIZEOF_LONG the flags identified by F_BYTE are
    not zero.  */
 #define INIT_BYTE_OBJECT(oop, value) \
-    ((oop)->flags |= sizeof (PTR) - ((value) & EMPTY_BYTES))
+    ((oop)->flags |= SIZEOF_LONG - ((value) & EMPTY_BYTES))
 
 
 /* Generally useful conversion functions */
 #define SIZE_TO_BYTES(size) \
-  ((size) * sizeof (PTR))
+  ((size) << LONG_SHIFT)
 
 #define BYTES_TO_SIZE(bytes) \
-  ((bytes) / sizeof (PTR))
+  ((bytes) >> LONG_SHIFT)
 
 
-/* integer conversions and some information on SmallIntegers.  */
+/* integer conversions and some information on SmallIntegers. */
 
 #define TO_INT(oop) \
-  ((intptr_t)(oop) >> 1)
+  ((long)(oop) >> 1)
 
 #define FROM_INT(i) \
-  (OOP)( ((intptr_t)(i) << 1) + 1)
+  (OOP)( ((long)(i) << 1) | 1)
 
-#define ST_INT_SIZE        ((sizeof (PTR) * 8) - 2)
+#define ST_INT_SIZE        ((SIZEOF_LONG * 8) - 2)
 #define MAX_ST_INT         ((1L << ST_INT_SIZE) - 1)
 #define MIN_ST_INT         ( ~MAX_ST_INT)
 #define INT_OVERFLOW(i)    ( (i) > MAX_ST_INT || (i) < MIN_ST_INT )
 #define OVERFLOWING_INT    (MAX_ST_INT + 1)
 
-#define INCR_INT(i)         ((OOP) (((intptr_t)i) + 2))	/* 1 << 1 */
-#define DECR_INT(i)         ((OOP) (((intptr_t)i) - 2))	/* 1 << 1 */
+#define INCR_INT(i)         ((OOP) (((long)i) + 2))	/* 1 << 1 */
+#define DECR_INT(i)         ((OOP) (((long)i) - 2))	/* 1 << 1 */
 
 /* Endian conversions, using networking functions if they do
    the correct job (that is, on 32-bit little-endian systems)
-   because they are likely to be optimized.  */
+   because they are likely to be optimized. */
 
-#if SIZEOF_INTPTR_T == 4
+#if SIZEOF_LONG == 4
 # if defined(WORDS_BIGENDIAN) || !defined (HAVE_INET_SOCKETS)
 #  define BYTE_INVERT(x) \
-        ((uintptr_t)((((uintptr_t)(x) & 0x000000ffU) << 24) | \
-                     (((uintptr_t)(x) & 0x0000ff00U) <<  8) | \
-                     (((uintptr_t)(x) & 0x00ff0000U) >>  8) | \
-                     (((uintptr_t)(x) & 0xff000000U) >> 24)))
+        ((unsigned long int)((((unsigned long int)(x) & 0x000000ffU) << 24) | \
+                             (((unsigned long int)(x) & 0x0000ff00U) <<  8) | \
+                             (((unsigned long int)(x) & 0x00ff0000U) >>  8) | \
+                             (((unsigned long int)(x) & 0xff000000U) >> 24)))
 # else
 #  define BYTE_INVERT(x) htonl((x))
 # endif
 
-#else /* SIZEOF_INTPTR_T == 8 */
+#else /* SIZEOF_LONG == 8 */
 # define BYTE_INVERT(x) \
-        ((uintptr_t)((((uintptr_t)(x) & 0x00000000000000ffU) << 56) | \
-                     (((uintptr_t)(x) & 0x000000000000ff00U) << 40) | \
-                     (((uintptr_t)(x) & 0x0000000000ff0000U) << 24) | \
-                     (((uintptr_t)(x) & 0x00000000ff000000U) <<  8) | \
-                     (((uintptr_t)(x) & 0x000000ff00000000U) >>  8) | \
-                     (((uintptr_t)(x) & 0x0000ff0000000000U) >> 24) | \
-                     (((uintptr_t)(x) & 0x00ff000000000000U) >> 40) | \
-                     (((uintptr_t)(x) & 0xff00000000000000U) >> 56)))
-#endif /* SIZEOF_INTPTR_T == 8 */
+        ((unsigned long)((((unsigned long)(x) & 0x00000000000000ffU) << 56) | \
+                         (((unsigned long)(x) & 0x000000000000ff00U) << 40) | \
+                         (((unsigned long)(x) & 0x0000000000ff0000U) << 24) | \
+                         (((unsigned long)(x) & 0x00000000ff000000U) <<  8) | \
+                         (((unsigned long)(x) & 0x000000ff00000000U) >>  8) | \
+                         (((unsigned long)(x) & 0x0000ff0000000000U) >> 24) | \
+                         (((unsigned long)(x) & 0x00ff000000000000U) >> 40) | \
+                         (((unsigned long)(x) & 0xff00000000000000U) >> 56)))
+#endif /* SIZEOF_LONG == 8 */
 
-/* The standard min/max macros...  */
+/* The standard min/max macros... */
 
 #ifndef ABS
 #define ABS(x) (x >= 0 ? x : -x)
@@ -444,7 +415,7 @@ extern OOP _gst_nil_oop, _gst_true_oop, _gst_false_oop;
 
    GCC 2.96 also introduced branch prediction hints for basic block
    reordering.  We use a shorter syntax than the wordy one that GCC
-   wants.  */
+   wants. */
 #if GNUC_PREREQ (2, 96)
 #define UNCOMMON(x) (__builtin_expect ((x) != 0, 0))
 #define COMMON(x)   (__builtin_expect ((x) != 0, 1))
@@ -525,7 +496,25 @@ extern OOP _gst_nil_oop, _gst_true_oop, _gst_false_oop;
 #include <sys/mman.h>
 #endif
 
-#include "stdintx.h"
+/* TODO: replace this cruft with autoconf tests that generate a stdint.h */
+#if defined HAVE_INTTYPES_H
+#include <inttypes.h>
+#elif defined HAVE_STDINT_H
+#include <stdint.h>
+#elif defined __GNUC__
+typedef long long int64_t;
+typedef unsigned long long uint64_t;
+#endif
+
+#if defined HAVE_INTTYPES_H || defined HAVE_STDINT_H
+#ifndef UINTMAX_C
+#define UINTMAX_C(n) n##UL
+#endif
+#ifndef UINTMAX_MAX
+#define UINTMAX_MAX ULONG_MAX
+#endif
+#endif
+
 #include "ansidecl.h"
 #include "signalx.h"
 #include "mathl.h"
@@ -539,18 +528,18 @@ extern OOP _gst_nil_oop, _gst_true_oop, _gst_false_oop;
 
 #include "tree.h"
 #include "input.h"
+#include "byte.h"
 #include "callin.h"
 #include "cint.h"
 #include "dict.h"
 #include "events.h"
 #include "gstpub.h"
 #include "heap.h"
+#include "comp.h"
+#include "interp.h"
 #include "lex.h"
 #include "lib.h"
 #include "oop.h"
-#include "byte.h"
-#include "comp.h"
-#include "interp.h"
 #include "opt.h"
 #include "save.h"
 #include "str.h"
@@ -559,8 +548,6 @@ extern OOP _gst_nil_oop, _gst_true_oop, _gst_false_oop;
 #include "xlat.h"
 #include "mpz.h"
 #include "print.h"
-#include "security.h"
-#include "match.h"
 
 /* Include this last, it has the bad habit of #defining printf
    and this fools gcc's __attribute__ (format) */
