@@ -32,33 +32,8 @@
  *
  ***********************************************************************/
 
-#include "gst.h"
 #include "gstpriv.h"
 
-#include <errno.h>
-#include <math.h>
-#include <stdio.h>
-#include "signalx.h"
-#include <sys/stat.h>
-#include <sys/types.h>
-#include <setjmp.h>
-#include "strspell.h"
-#include "mathl.h"
-#include "socketx.h"
-
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
-#ifdef STDC_HEADERS
-#include <string.h>
-#include <stdlib.h>
-#endif /* STDC_HEADERS */
-
-#ifdef HAVE_EXECINFO_H
-#include <execinfo.h>
-#endif
-
 
 
 /* The local regs concept hopes, by caching the values of IP and SP in
@@ -315,7 +290,7 @@ static mst_Boolean verbose_exec_tracing = false;
 
 /* Locates in the ProcessorScheduler's process lists and returns the
    highest priority process different from the current process.  */
-static OOP highest_priority_process (void) FN_PURE;
+static OOP highest_priority_process (void) ATTRIBUTE_PURE;
 
 /* Remove the head of the given list (a Semaphore is a subclass of
    LinkedList) and answer it. */
@@ -406,24 +381,24 @@ static mst_Boolean resume_process (OOP processOOP,
 
 /* Answer whether PROCESSOOP is ready to execute (neither terminating,
    nor suspended, nor waiting on a semaphore).  */
-static mst_Boolean is_process_ready (OOP processOOP) FN_PURE;
+static mst_Boolean is_process_ready (OOP processOOP) ATTRIBUTE_PURE;
 
 /* Answer whether any processes are queued in the PROCESSLISTOOP
    (which can be a LinkedList or a Semaphore).  */
-static inline mst_Boolean is_empty (OOP processListOOP) FN_PURE;
+static inline mst_Boolean is_empty (OOP processListOOP) ATTRIBUTE_PURE;
 
 /* Answer whether the processs is terminating, that is, it does not
    have an execution context to resume execution from. */
-static inline mst_Boolean is_process_terminating (OOP processOOP) FN_PURE;
+static inline mst_Boolean is_process_terminating (OOP processOOP) ATTRIBUTE_PURE;
 
 /* Answer the process that is scheduled to run (that is, the
    executing process or, if any, the process that is scheduled
    to start execution at the next sequence point. */
-static inline OOP get_scheduled_process (void) FN_PURE;
+static inline OOP get_scheduled_process (void) ATTRIBUTE_PURE;
 
 /* Answer the active process (that is, the process that executed
    the last bytecode. */
-static inline OOP get_active_process (void) FN_PURE;
+static inline OOP get_active_process (void) ATTRIBUTE_PURE;
 
 /* Create a new Semaphore OOP with SIGNALS signals on it and return it. */
 static inline OOP semaphore_new (int signals);
@@ -491,7 +466,7 @@ static void unwind_context (void);
 /* Used to help minimize the number of primitives used to control the
    various debugging flags, this routine maps the variable's INDEX to the
    address of a boolean debug flag, which it returns. */
-static inline mst_Boolean *bool_addr_index (int index) FN_PURE;
+static inline mst_Boolean *bool_addr_index (int index) ATTRIBUTE_PURE;
 
 /* Check whether it is true that sending SENDSELECTOR to RECEIVER
    accepts NUMARGS arguments.  Note that the RECEIVER is only used to
@@ -503,7 +478,7 @@ static inline mst_Boolean *bool_addr_index (int index) FN_PURE;
    _gst_nil_oop or directly call _gst_selector_num_args. */
 static inline mst_Boolean check_send_correctness (OOP receiver,
 						  OOP sendSelector,
-						  int numArgs) FN_PURE;
+						  int numArgs) ATTRIBUTE_PURE;
 
 /* Unwind the contexts up until the caller of the method that
    created the block context, no matter how many levels of message
@@ -548,6 +523,42 @@ static RETSIGTYPE interrupt_handler (int sig);
 static RETSIGTYPE preempt_smalltalk_process (int sig);
 #endif
 
+/* This macro acts as a block statement (if, for, while); it
+   accepts a pointer to an interp_jmp_buf and executes its body
+   so that the current process is suspended and SIGINT breaks
+   out of it.  */
+#define PROTECT_CURRENT_PROCESS_WITH(jb) \
+  for ((jb)->next = reentrancy_jmp_buf, \
+       (jb)->suspended = 0, \
+       (jb)->processOOP = get_active_process (), \
+       _gst_register_oop ((jb)->processOOP), \
+       reentrancy_jmp_buf = (jb), \
+       in_interpreter = false; \
+       !in_interpreter; \
+       in_interpreter = true, \
+       _gst_unregister_oop ((jb)->processOOP), \
+       reentrancy_jmp_buf = reentrancy_jmp_buf->next) \
+    if (setjmp (reentrancy_jmp_buf->jmpBuf) != 0) \
+      continue; \
+    else \
+
+/* This macro acts as a block statement (if, for, while); it
+   accepts a pointer to an interp_jmp_buf and executes its body
+   so that the current process is not suspended (like in
+   asynchronous C call-outs) but SIGINT breaks out of it.  */
+#define PROTECT_FROM_INTERRUPT_WITH(jb) \
+  for ((jb)->next = reentrancy_jmp_buf, \
+       (jb)->suspended = 0, \
+       (jb)->processOOP = _gst_nil_oop, \
+       reentrancy_jmp_buf = (jb), \
+       in_interpreter = false; \
+       !in_interpreter; \
+       in_interpreter = true, \
+       reentrancy_jmp_buf = reentrancy_jmp_buf->next) \
+    if (setjmp (reentrancy_jmp_buf->jmpBuf) != 0) \
+      continue; \
+    else \
+
 /* Pick a process that is the highest-priority process different from
    the currently executing one, and schedule it for execution after
    the first sequence points. */
@@ -2349,7 +2360,7 @@ interrupt_handler (int sig)
 	{
           PTR array[11];
           size_t size = backtrace (array, 11);
-          backtrace_symbols_fd (array + 1, size, STDERR_FILENO);
+          backtrace_symbols_fd (array + 1, size - 1, STDERR_FILENO);
         }
 #endif
     }
@@ -2472,18 +2483,6 @@ parse_stream_with_protection ()
 {
   interp_jmp_buf localJmpBuf;
 
-  localJmpBuf.next = reentrancy_jmp_buf;
-  localJmpBuf.suspended = 0;
-  localJmpBuf.processOOP = get_active_process ();
-  _gst_register_oop (localJmpBuf.processOOP);
-
-  reentrancy_jmp_buf = &localJmpBuf;
-  in_interpreter = false;
-
-  if (setjmp (reentrancy_jmp_buf->jmpBuf) == 0)
+  PROTECT_CURRENT_PROCESS_WITH (&localJmpBuf)
     _gst_parse_stream ();
-
-  in_interpreter = true;
-  _gst_unregister_oop (localJmpBuf.processOOP);
-  reentrancy_jmp_buf = reentrancy_jmp_buf->next;
 }
