@@ -62,12 +62,12 @@ enum
 
 /* These macros are used to quickly compute the number of words needed
    for a context with a maximum allowable stack depth of DEPTH.  */
-#define FIXED_CTX_SIZE	(sizeof(struct gst_method_context) / sizeof(PTR) - 1)
+#define FIXED_CTX_SIZE	(sizeof(struct gst_method_context) / sizeof(long) - 1)
 #define CTX_SIZE(depth) (((depth) << DEPTH_SCALE) + FIXED_CTX_SIZE)
 
 #define DUMMY_NATIVE_IP FROM_INT(0)
 
-/* The structure of execution context objects.  */
+/* The structure of execution context objects. */
 typedef struct gst_context_part
 {
   OBJ_HEADER;
@@ -93,7 +93,7 @@ typedef struct gst_method_context
 				   stack */
   OOP receiver;			/* the receiver OOP */
   OOP method;			/* the method that we're executing */
-  intptr_t flags;		/* must be an int to distinguish
+  long flags;			/* must be an int to distinguish
 				   gst_compiled_block/gst_method_context 
 				 */
   OOP contextStack[1];
@@ -121,7 +121,7 @@ typedef struct gst_method_context
 /* Answer whether execution started from this context (this kind
    of MethodContext is used to mark call-ins from C to Smalltalk
    and is placed on top of the context that was executing at the
-   time of the call-in, and is the parent of the called-in method).  */
+   time of the call-in, and is the parent of the called-in method). */
 #define MCF_IS_EXECUTION_ENVIRONMENT  8
 
 
@@ -141,7 +141,7 @@ typedef struct gst_block_context
 }
  *gst_block_context;
 
-/* The structure of various objects related to the process system.  */
+/* The structure of various objects related to the process system. */
 typedef struct gst_semaphore
 {
   OBJ_HEADER;
@@ -183,20 +183,18 @@ typedef struct gst_processor_scheduler
   OOP activeProcess;
   OOP idleTasks;
   OOP processTimeslice;
-  OOP gcSemaphore;
-  OOP gcArray;
 }
  *gst_processor_scheduler;
 
 /* Some performance counters from the interpreter: these
-   count the number of special returns.  */
+   count the number of special returns. */
 extern unsigned long _gst_literal_returns, _gst_inst_var_returns,
   _gst_self_returns;
 
-/* The number of primitives executed.  */
+/* The number of primitives executed. */
 extern unsigned long _gst_primitives_executed;
 
-/* The number of bytecodes executed.  */
+/* The number of bytecodes executed. */
 extern unsigned long _gst_bytecode_counter;
 
 /* The number of method cache misses */
@@ -207,35 +205,35 @@ extern unsigned long _gst_sample_counter;
 
 /* If this is true, for each byte code that is executed, we print on
    stdout the byte index within the current gst_compiled_method and a
-   decoded interpretation of the byte code.  */
+   decoded interpretation of the byte code. */
 extern mst_Boolean _gst_execution_tracing;
 
 /* When this is true, and an interrupt occurs (such as SIGSEGV),
    Smalltalk will terminate itself by making a core dump (normally it
-   produces a backtrace).  */
+   produces a backtrace). */
 extern mst_Boolean _gst_make_core_file;
 
 /* When true, this indicates that there is no top level loop for
-   control to return to, so it causes the system to exit.  */
+   control to return to, so it causes the system to exit. */
 extern mst_Boolean _gst_non_interactive;
 
 /* The OOP for a gst_compiled_method or gst_compiled_block that is the
-   currently executing method.  */
+   currently executing method. */
 extern OOP _gst_this_method;
 
 /* Physical address of the base of the method temporary variables.
    Typically a small number of bytes (multiple of 4 since it points to
-   OOPs) lower than sp.  */
+   OOPs) lower than sp. */
 extern OOP *_gst_temporaries;
 
-/* Physical address of the base of the method literals.  */
+/* Physical address of the base of the method literals. */
 extern OOP *_gst_literals;
 
-/* An OOP that is the current receiver of the current message.  */
+/* An OOP that is the current receiver of the current message. */
 extern OOP _gst_self;
 
 /* A gst_block_context or gst_method_context that indicates the
-   context that the interpreter is currently running in.  */
+   context that the interpreter is currently running in. */
 extern OOP _gst_this_context_oop;
 
 /* The type used to hold the instruction pointer.  For the JIT, this
@@ -244,8 +242,8 @@ extern OOP _gst_this_context_oop;
    SmallInteger and can be stored in the returnIP field of the context
    objects); for the interpreter, this is the physical address of the
    next executed bytecode (note: the global is usually synchronized at
-   sequence points only).  */
-#ifdef ENABLE_JIT_TRANSLATION
+   sequence points only). */
+#ifdef USE_JIT_TRANSLATION
 typedef int ip_type;
 extern char *native_ip;
 #else /* plain bytecode interpreter */
@@ -256,7 +254,7 @@ typedef gst_uchar *ip_type;
 extern ip_type ip;
 
 /* Set to true when some special action must be done at the next
-   sequence point.  */
+   sequence point. */
 extern volatile mst_Boolean _gst_except_flag;
 
 /* Create a new Process on the top of the stack, which is specially
@@ -298,7 +296,7 @@ extern void _gst_send_message_internal (OOP sendSelector,
 			      OOP receiver,
 			      OOP method_class);
 
-/* Prepare the data structures held by the interpreter.  */
+/* Prepare the data structures held by the interpreter. */
 extern void _gst_init_interpreter (void);
 
 /* Reset the fast allocator for context objects, telling it that
@@ -311,7 +309,7 @@ extern void _gst_empty_context_pool (void);
    operation, the correct way to signal a semaphore is not to send
    #signal to the object but, rather, to use this function.  The
    signal request will be processed as soon as the next sequence point
-   is reached.  */
+   is reached. */
 extern void _gst_async_signal (OOP semaphoreOOP);
 
 /* Signal SEMAPHOREOOP so that one of the processes waiting on that
@@ -319,7 +317,7 @@ extern void _gst_async_signal (OOP semaphoreOOP);
    Smalltalk call-in is not an atomic operation, the correct way to
    signal a semaphore is not to send #signal to the object but,
    rather, to use this function.  The signal request will be processed
-   as soon as the next sequence point is reached.  */
+   as soon as the next sequence point is reached. */
 extern void _gst_async_signal_and_unregister (OOP semaphoreOOP);
 
 /* Invalidate all the cached CompiledMethod lookups.  This does NOT
@@ -331,7 +329,7 @@ extern void _gst_invalidate_method_cache (void);
 extern void _gst_show_backtrace (void);
 
 /* Trap the signals that we care about, basically SIGBUS and
-   SIGSEGV.  */
+   SIGSEGV. */
 extern void _gst_init_signals (void);
 
 /* Store the context of the VM registers into the currently executing
@@ -347,12 +345,12 @@ extern void _gst_fixup_object_pointers (void);
 /* Complementary to _gst_fixup_object_pointers, this function picks
    the relative addresses stored in the current context and uses
    them to adjust the VM registers after the heap is compacted or
-   grown.  */
+   grown. */
 extern void _gst_restore_object_pointers (void);
 
 /* This runs before every evaluation (_gst_execute_statements) and
    before GC turned on.  It creates an initial process if no process
-   is ready to run or if no process has been created yet.  */
+   is ready to run or if no process has been created yet. */
 extern void _gst_init_process_system (void);
 
 /* These function mark or copy all the objects that the interpreter keeps in
@@ -372,7 +370,7 @@ extern void _gst_mark_processor_registers (void);
 extern void _gst_copy_processor_registers (void);
 
 /* Print the current state of the lists of ready to run processes for
-   each priority, for debugging purposes.  */
+   each priority, for debugging purposes. */
 extern void _gst_print_process_state (void);
 
 /* Sanity check the process lists that the sole instance of ProcessorScheduler
@@ -380,7 +378,7 @@ extern void _gst_print_process_state (void);
 extern void _gst_check_process_state (void);
 
 /* Print the objects currently on the stack, for debugging
-   purposes.  */
+   purposes. */
 extern void _gst_show_stack_contents (void);
 
 /* Called after the mark phase, but before the sweep phase, so that if
@@ -391,12 +389,9 @@ extern void _gst_show_stack_contents (void);
    also if the translation to native code for the method is garbage
    collected.  In particular, this function is called *after* the
    unused method translations are marked as such, and *before* they
-   are actually freed.  */
+   are actually freed. */
 extern void _gst_validate_method_cache_entries (void);
 
-/* Terminate execution of the given PROCESSOOP.  */
-extern void _gst_terminate_process (OOP processOOP);
-
 /* Similar to _gst_send_message_internal, but forces the specified
    CompiledMethod to be sent.  If it is not valid for the current
    receiver, well, you are looking for trouble and well deserve it.
@@ -409,13 +404,9 @@ extern void _gst_send_method (OOP methodOOP);
    correct way to signal a semaphore is not to send the wait method to
    the object but, rather, to use _gst_sync_wait.  The `sync' in the
    name of this function distinguishes it from _gst_async_signal, in
-   that it cannot be called from within a signal handler.  */
+   that it cannot be called from within a signal handler. */
 extern void _gst_sync_wait (OOP semaphoreOOP);
 
-/* Take a CompiledBlock and turn it into a BlockClosure that references
-   thisContext as its static link.  */
-extern OOP _gst_make_block_closure (OOP blockOOP);
-
 /************************************************* PRIMITIVES ****************/
 
 /* The type for a routine providing the definition for one or more
@@ -426,15 +417,15 @@ extern OOP _gst_make_block_closure (OOP blockOOP);
    to invoke the primitive.  Such a function must execute a primitive,
    aided in the choice of which by the user-defined parameter ID,
    popping NUMARGS methods off the stack if they succeed.  */
-typedef intptr_t (*primitive_func) (int primitive, 
-				    volatile int numArgs);
+typedef long (*primitive_func) (int primitive, 
+				volatile int numArgs);
 
-/* Table of primitives, including a primitive and its attributes.  */
+/* Table of primitives, including a primitive and its attributes. */
 typedef struct prim_table_entry
 {
-  const char *name;
+  char *name;
+  long attributes;
   primitive_func func;
-  int attributes;
   int id;
 }
 prim_table_entry;
@@ -453,14 +444,14 @@ prim_table_entry;
    system.  This function invokes the proper primitive_func with the
    correct id and the same NUMARGS and METHODOOP with which it was
    invoked.  */
-extern intptr_t _gst_execute_primitive_operation (int primitive,
-					          volatile int numArgs);
+extern long _gst_execute_primitive_operation (int primitive,
+					      volatile int numArgs);
 
 /* This can be used to obtain information on a particular primitive
    operations in the GNU Smalltalk system.  */
 extern prim_table_entry * _gst_get_primitive_attributes (int primitive) ATTRIBUTE_PURE;
 
-/* Initialize the table of primitives.  */
+/* Initialize the table of primitives. */
 extern void _gst_init_primitives ();
 
 #endif /* GST_INTERP_H */
