@@ -39,7 +39,7 @@
 #endif
 
 
-/* Define to debug the getopt code.  */
+/* Define to debug the getopt code. */
 /* #define DEBUG_GETOPT */
 
 #ifdef atarist
@@ -71,7 +71,7 @@ static const char help_text[] =
   "\n   -c --core-dump\t\t Dump core on fatal signal"
   "\n   -d --declaration-trace-user\t Trace compilation of files on the command line"
   "\n   -D --declaration-trace-all\t Trace compilation of all loaded files"
-#ifndef ENABLE_JIT_TRANSLATION
+#ifndef USE_JIT_TRANSLATION
   "\n   -e --execution-trace-user\t Trace execution of files on the command line"
   "\n   -E --execution-trace-all\t Trace execution of all loaded files"
 #endif
@@ -80,6 +80,8 @@ static const char help_text[] =
   "\n   -i --rebuild-image\t\t Ignore the image file; rebuild it from scratch"
   "\n   -I --image-file file\t\t Instead of `gst.im', use `file' as the image\n\t\t\t\t file, and ignore the kernel files' timestamps\n"
   "\n   -K --kernel-file file\t Make file's path relative to the image path."
+  "\n   -l --log-changes\t\t Log changes in `change-log.st'"
+  "\n   -L --log-file file\t\t Log changes in file `file'"
   "\n   -p --emacs-mode\t\t Execute as a `process' (from within Emacs)"
   "\n   -q --quiet --silent\t\t Do not print execution information"
   "\n   -Q --no-messages\t\t Run Smalltalk with -q -g and no startup banner"
@@ -114,7 +116,7 @@ static const struct option long_options[] = {
   {"core-dump", 0, 0, 'c'},
   {"declaration-trace-user", 0, 0, 'd'},
   {"declaration-trace-all", 0, 0, 'D'},
-#ifndef ENABLE_JIT_TRANSLATION
+#ifndef USE_JIT_TRANSLATION
   {"execution-trace-user", 0, 0, 'e'},
   {"execution-trace-all", 0, 0, 'E'},
 #endif
@@ -123,6 +125,8 @@ static const struct option long_options[] = {
   {"rebuild-image", 0, 0, 'i'},
   {"image-file", 1, 0, 'I'},
   {"kernel-file", 1, 0, 'K'},
+  {"log-changes", 0, 0, 'l'},
+  {"log-file", 1, 0, 'L'},
   {"emacs-mode", 0, 0, 'p'},
   {"quiet", 0, 0, 'q'},
   {"no-messages", 0, 0, 'Q'},
@@ -144,27 +148,27 @@ static const struct option long_options[] = {
 mst_Boolean _gst_quiet_execution;
 
 /* Set by cmd line flag.  If true, Smalltalk is more verbose about
-   what it's doing.  */
+   what it's doing. */
 mst_Boolean _gst_verbose = false;
 
 /* These contain the default path that was picked (after looking at the
-   environment variables) for the kernel files and the image.  */
-const char *_gst_kernel_file_path, *_gst_image_file_path;
+   environment variables) for the kernel files and the image. */
+char *_gst_kernel_file_default_path, *_gst_image_file_default_path;
 
 /* This is the name of the binary image to load.  If it is not NULL after the
    command line is parsed, the checking of the dates of the kernel source files
    against the image file date is overridden.  If it is NULL, it is set to
-   default_image_name.  */
-const char *_gst_binary_image_name = NULL;
+   default_image_name. */
+char *_gst_binary_image_name = NULL;
 
 /* This is used by the callin functions to auto-initialize Smalltalk.
    When it's not true, initialization needs to be performed.  It's set
-   to true by gst_init_smalltalk().  */
+   to true by gst_init_smalltalk(). */
 mst_Boolean _gst_smalltalk_initialized = false;
 
 /* This is used to avoid doing complicated things (currently, this
    includes call-ins before and after _gst_execute_statements) before
-   the system is ready to do them.  */
+   the system is ready to do them. */
 mst_Boolean _gst_kernel_initialized = false;
 
 /* If true, even both kernel and user method definitions are shown as
@@ -172,7 +176,7 @@ mst_Boolean _gst_kernel_initialized = false;
 mst_Boolean _gst_trace_kernel_declarations;
 
 /* If true, execution tracing is performed when loading kernel method
-   definitions.  */
+   definitions. */
 mst_Boolean _gst_trace_kernel_execution;
 
 /* This is TRUE if we are doing regression testing, and causes
@@ -191,7 +195,7 @@ mst_Boolean _gst_regression_testing;
 /* Parse the Smalltalk source code contained in FILENAME.  If QUIET
    is true, for no reason show messages about the execution state (this
    is turned on while loading kernel files).  */
-static mst_Boolean process_file (const char *fileName);
+static mst_Boolean process_file (char *fileName);
 
 /* Parse the Smalltalk source code read from stdin.  */
 static void process_stdin ();
@@ -200,7 +204,7 @@ static void process_stdin ();
    _gst_binary_image_name.  This is good is the image file is local
    and newer than all of the kernel files, or if the image file is
    global, newer than all of the global kernel files, and no local
-   kernel file is found.  */
+   kernel file is found. */
 static mst_Boolean ok_to_load_binary (void);
 
 /* Attempts to find a viable kernel Smalltalk file (.st file).  First
@@ -236,48 +240,45 @@ static int load_standard_files (void);
 /* Sets up the paths for the kernel source directory and for where the
    saved Smalltalk binary image lives.  Uses environment variables
    SMALLTALK_KERNEL and SMALLTALK_IMAGE if they are set, otherwise
-   uses the paths passed by the makefiles.  */
+   uses the paths passed by the makefiles. */
 static void init_paths (void);
 
 /* This routine scans the command line arguments, accumulating
-   information and setting flags.  */
+   information and setting flags. */
 static int parse_args (int argc,
-		       const char **argv);
+		       char **argv);
 
 /* These functions load the per-user customization files, respectively
    .stinit (loaded at every startup) and .stpre (loaded before a local
-   image is saved.  */
+   image is saved. */
 static void load_user_init_file (void);
 static void load_user_pre_image_file (void);
 
 /* Set by command line flag.  When true, Smalltalk saves a snapshot after
-   loading the files on the command line, before exiting.  */
+   loading the files on the command line, before exiting. */
 static mst_Boolean snapshot_after_load = false;
 
 /* Whether SMALLTALK_IMAGE is set (only if it is set, .stpre is loaded) */
 static mst_Boolean is_local_image;
 
-/* Usually the same as _gst_image_file_path.  */
-static char *default_image_path;
-
-/* The default_image_path followed by /gst.im */
+/* The _gst_image_file_default_path followed by /gst.im */
 static char *default_image_name;
 
 /* Set by command line flag.  When true, Smalltalk does not print any
    sort of version banner at the startup of the interpreter.  This
    makes Smalltalk usable as a filter, or as a pure producer of
-   content (such as the main program in a cgi-bin script).  */
+   content (such as the main program in a cgi-bin script). */
 static mst_Boolean no_errors = false;
 
 /* If true, skip date checking of kernel files vs. binary image; pretend
-   that binary image does not exist.  */
+   that binary image does not exist. */
 static mst_Boolean ignore_image;
 
 /* The complete list of "kernel" class and method definitions.  Each
    of these files is loaded, in the order given below.  Their last
    modification dates are compared against that of the image file; if
    any are newer, the image file is ignored, these files are loaded,
-   and a new image file is created.  */
+   and a new image file is created. */
 static const char *standard_files[] = {
   "Builtins.st",
   "Object.st",
@@ -401,21 +402,21 @@ static const char *standard_files[] = {
 };
 
 /* The argc and argv that are passed to libgst via gst_smalltalk_args. 
-   The default is passing no parameters.  */
-static const char *smalltalk_arg_vec[] = { "gst", NULL };
+   The default is passing no parameters. */
+static char *smalltalk_arg_vec[] = { "gst", NULL };
 static int smalltalk_argc = 0;
-static const char **smalltalk_argv = smalltalk_arg_vec;
+static char **smalltalk_argv = smalltalk_arg_vec;
 
 /* The argc and argv that are made available to Smalltalk programs
-   through the -a option.  */
+   through the -a option. */
 int _gst_smalltalk_passed_argc = 0;
-const char **_gst_smalltalk_passed_argv = NULL;
+char **_gst_smalltalk_passed_argv = NULL;
 
 
 
 void
 gst_smalltalk_args (int argc,
-		    const char **argv)
+		    char **argv)
 {
   smalltalk_argc = argc;
   smalltalk_argv = argv;
@@ -481,21 +482,24 @@ gst_init_smalltalk (void)
 
       if (!loadBinary
           && !is_local_image
-          && !_gst_file_is_writeable (_gst_image_file_path))
+          && !_gst_file_is_writeable (_gst_image_file_default_path))
         {
+          xfree (_gst_binary_image_name);
+
           is_local_image = true;
-          xfree (default_image_path);
-          _gst_image_file_path = default_image_path = _gst_get_cur_dir_name ();
+          xfree (_gst_image_file_default_path);
+          _gst_image_file_default_path = _gst_get_cur_dir_name ();
           _gst_binary_image_name = "gst.im";
           loadBinary = !ignore_image && ok_to_load_binary();
         }
     }
 
   /* Compute the actual path of the image file */
-  _gst_image_file_path = p =
+  xfree (_gst_image_file_default_path);
+
+  p = _gst_image_file_default_path =
     _gst_get_full_file_name (_gst_binary_image_name);
-  
-  p += strlen (_gst_image_file_path);
+  p += strlen (_gst_image_file_default_path);
 #if defined(MSDOS) || defined(WIN32) || defined(__OS2__)
   while (*--p != '/' && *p != '\\');
 #else
@@ -505,8 +509,9 @@ gst_init_smalltalk (void)
 
   if (loadBinary && _gst_load_from_file (_gst_binary_image_name))
     {
-      _gst_init_interpreter ();
+      _gst_init_changes_stream ();
       _gst_init_compiler ();
+      _gst_init_interpreter ();
       _gst_init_vmproxy ();
     }
   else
@@ -523,11 +528,13 @@ gst_init_smalltalk (void)
       _gst_init_oop_table (INITIAL_OOP_TABLE_SIZE);
       _gst_init_mem_default ();
       _gst_init_dictionary ();
-      _gst_init_interpreter ();
       _gst_init_compiler ();
+      _gst_init_interpreter ();
       _gst_init_vmproxy ();
 
+      _gst_reset_changes_file ();
       _gst_install_initial_methods ();
+      _gst_init_changes_stream ();
 
       result = load_standard_files ();
       if (!result)
@@ -594,49 +601,41 @@ void
 init_paths (void)
 {
   char *currentDirectory = _gst_get_cur_dir_name ();
-  const char *kernel_path, *image_path;
 
-  kernel_path = (char *) getenv ("SMALLTALK_KERNEL");
-  image_path = (char *) getenv ("SMALLTALK_IMAGE");
+  _gst_kernel_file_default_path = (char *) getenv ("SMALLTALK_KERNEL");
+  _gst_image_file_default_path = (char *) getenv ("SMALLTALK_IMAGE");
   is_local_image = true;
 
-  if (!kernel_path
-      || !_gst_file_is_readable (kernel_path))
+  if (!_gst_kernel_file_default_path
+      || !_gst_file_is_readable (_gst_kernel_file_default_path))
+    asprintf (&_gst_kernel_file_default_path, "%s/kernel",
+	      currentDirectory);
+
+  if (!_gst_image_file_default_path
+      || !_gst_file_is_readable (_gst_image_file_default_path))
     {
-      char *kernel_file_path;
-      asprintf (&kernel_file_path, "%s/kernel",
-	        currentDirectory);
+      _gst_image_file_default_path = IMAGE_PATH;
 
-      _gst_kernel_file_path = kernel_file_path;
-    }
-  else
-    _gst_kernel_file_path = xstrdup (kernel_path);
-
-  if (!image_path
-      || !_gst_file_is_readable (image_path))
-    {
-      image_path = IMAGE_PATH;
-
-      if (_gst_file_is_readable (image_path))
+      if (_gst_file_is_readable (_gst_image_file_default_path))
 	/* Found in the standard image path.  Apply this kludge so
 	   that OSes such as Windows and MS-DOS which have no concept 
-	   of home directories always load the .stpre file.  */
+	   of home directories always load the .stpre file. */
 	is_local_image = (((char *) getenv ("HOME")) == NULL);
       else
-	image_path = currentDirectory;
+	_gst_image_file_default_path = currentDirectory;
     }
 
   asprintf (&default_image_name, "%s/gst.im",
-	    image_path);
+	    _gst_image_file_default_path);
 
-  _gst_image_file_path = default_image_path = xstrdup (image_path);
+  _gst_image_file_default_path = xstrdup (_gst_image_file_default_path);
   xfree (currentDirectory);
 }
 
 mst_Boolean
 ok_to_load_binary (void)
 {
-  time_t imageFileTime;
+  long imageFileTime;
   const char **fileNames;
   char fullFileName[MAXPATHLEN], *home, preImageFileName[MAXPATHLEN];
 
@@ -652,7 +651,7 @@ ok_to_load_binary (void)
 	{
 	  /* file lives locally but the image doesn't -- bad semantics.
 	     Note that if SOME of the files are local and the image file
-	     is local, it is good.  */
+	     is local, it is good. */
 	  return (false);
 	}
       if (imageFileTime < _gst_get_file_modify_time (fullFileName))
@@ -712,14 +711,14 @@ find_kernel_file (const char *fileName,
       return (true);
     }
 
-  sprintf (fullFileName, "%s/%s", _gst_kernel_file_path,
+  sprintf (fullFileName, "%s/%s", _gst_kernel_file_default_path,
 	   fileName);
   if (_gst_file_is_readable (fullFileName))
     {
       char systemFileName[256];
       sprintf (systemFileName, KERNEL_PATH "/%s", fileName);
       /* If this file and the system file are the same, consider the
-         file as a system file instead.  */
+         file as a system file instead. */
       return (!_gst_file_is_readable (systemFileName) ||
 	      _gst_get_file_modify_time (fullFileName) !=
 	      _gst_get_file_modify_time (systemFileName));
@@ -776,7 +775,7 @@ process_stdin ()
 }
 
 mst_Boolean
-process_file (const char *fileName)
+process_file (char *fileName)
 {
   int fd;
 
@@ -796,12 +795,12 @@ process_file (const char *fileName)
 
 int
 parse_args (int argc,
-	    const char **argv)
+	    char **argv)
 {
-  const char **av = argv;
+  char **av = argv;
   int ch, prev = 0, prevOptind = -1;
 
-#ifndef ENABLE_DYNAMIC_TRANSLATION
+#ifndef USE_DYNAMIC_TRANSLATION
 # define OPTIONS "-acdDeEghHiI:K:lL:pQqrsSvVy"
 #else
 # define OPTIONS "-acdDghHiI:K:lL:pQqrsSvVy"
@@ -811,7 +810,7 @@ parse_args (int argc,
   opterr = 1;
 
   while ((ch =
-	  getopt_long (argc, (char **) argv, OPTIONS, long_options, NULL)) != -1)
+	  getopt_long (argc, argv, OPTIONS, long_options, NULL)) != -1)
     {
 #undef OPTIONS
 
@@ -825,7 +824,7 @@ parse_args (int argc,
 	  /* The first option after -a was not introduced by a minus,
 	     so getopt_long passed it to us as '\1'.  Undo the effect
 	     of this last getopt_long, so that this option is actually
-	     passed to Smalltalk.  */
+	     passed to Smalltalk. */
 	  optind--;
 	  break;
 	}
@@ -844,7 +843,7 @@ parse_args (int argc,
 	case 'd':
 	  _gst_declare_tracing = true;
 	  break;
-#ifndef ENABLE_JIT_TRANSLATION
+#ifndef USE_JIT_TRANSLATION
 	case 'E':
 	  _gst_trace_kernel_execution = true;		/* fall thru */
 	case 'e':
@@ -882,13 +881,26 @@ parse_args (int argc,
 	  _gst_yydebug = 1;
 	  break;
 
+	  /* ??? Is there some better place for this to be? Canonical
+	     places seem to lose in that canonical places are often
+	     write-protected, which is ok for extant changes (like in a 
+	     saved system), but not for new changes by the user.
+	     Again, the notion of "the" user's home area is kind of
+	     random if there are multiple snapshots, since each
+	     snapshot should be associated with a particular changes
+	     file for which it is relevant. We use the current
+	     directory and allow to override with the -L option. */
+	case 'l':
+	  _gst_change_file_name = "change-log.st";
+	  break;
+
 	case 'a':
 	  /* "Officially", the C command line ends here.  The Smalltalk 
 	     command line, instead, starts right after the parameter
 	     containing -a. -a is handled specially by the code that
 	     tests the prev and prevOptind variables, so that ./gst -aI 
 	     xxx yyy for example interprets xxx as the image to be
-	     loaded.  */
+	     loaded. */
 	  break;
 
 	case 'I':
@@ -902,12 +914,15 @@ parse_args (int argc,
 
 	case 'K':
 	  {
-	    char *file;
-	    asprintf (&file, "%s/%s", _gst_image_file_path, optarg);
-	    *++av = file;
+	    asprintf (++av, "%s/%s", _gst_image_file_default_path,
+		      optarg);
 	    break;
 	  }
 
+	case 'L':
+	  _gst_change_file_name = optarg;
+	  break;
+
 	case 'v':
 	  printf (copyright_and_legal_stuff_text, VERSION, KERNEL_PATH,
 		  IMAGE_PATH);
@@ -926,7 +941,7 @@ parse_args (int argc,
       if (prev != 'a')
 	{
 	  /* Track whether we have found a '-a' argument, by updating
-	     prev only until we find it.  */
+	     prev only until we find it. */
 	  prev = ch;
 
 	}
@@ -934,7 +949,7 @@ parse_args (int argc,
 	{
 	  /* We had found a '-a' argument and we ate all the remaining
 	     options in the element of argv that contained -a.  All the 
-	     remaining arguments are passed to Smalltalk.  */
+	     remaining arguments are passed to Smalltalk. */
 	  break;
 	}
       prevOptind = optind;
