@@ -28,11 +28,7 @@
  *
  ***********************************************************************/
 
-#include "gst.h"
 #include "gstpriv.h"
-#include <stdlib.h>
-#include <math.h>
-#include "mathl.h"
 
 #if HAVE_GMP
 #include <gmp.h>
