@@ -51,8 +51,8 @@
 
 /* Define this flag to disable blacking of grey pages (that is, the
    entire oldspace is scanned to look for reachable newspace objects).
-   This is also necessary to run valgrind on GNU Smalltalk.  */
-#define NO_SIGSEGV_HANDLING
+   This is also necessary to run valgrind on GNU Smalltalk. */
+/* #define NO_SIGSEGV_HANDLING */
 
 /* Define this flag to turn on debugging code for oldspace management */
 /* #define MMAN_DEBUGGING */
@@ -68,16 +68,16 @@
 
 
 
-/* These are the memory areas within which we mantain object data.  */
+/* These are the memory areas within which we mantain object data. */
 heap _gst_oop_heap;
 
 /* These are the real OOPS for nil, true, and false */
 OOP _gst_nil_oop, _gst_true_oop, _gst_false_oop;
 
-/* This is true to show a message whenever a GC happens.  */
+/* This is true to show a message whenever a GC happens. */
 mst_Boolean _gst_gc_message = true;
 
-/* This is != 0 in the middle of a GC.  */
+/* This is != 0 in the middle of a GC. */
 int _gst_gc_running = 0;
 
 /* This vector holds the storage for all the Character objects in the
@@ -87,7 +87,7 @@ int _gst_gc_running = 0;
 static struct gst_character _gst_char_object_table[NUM_CHAR_OBJECTS];
 
 /* This is "nil" object in the system.  That is, the single instance
-   of the UndefinedObject class, which is called "nil".  */
+   of the UndefinedObject class, which is called "nil". */
 static struct gst_undefined_object _gst_nil_object;
 
 /* These represent the two boolean objects in the system, true and
@@ -99,7 +99,7 @@ static struct gst_boolean _gst_boolean_objects[2];
 /* This variable represents information about the memory space.
    _gst_mem holds the required information: basically the
    pointer to the base and top of the space, and the pointers into it
-   for allocation and copying.  */
+   for allocation and copying. */
 struct memory_space _gst_mem;
 
 /* Data to compute the statistics in _gst_mem.  */
@@ -113,7 +113,7 @@ struct statistical_data
 
 
 /* Allocates a table for OOPs of SIZE bytes, and store pointers to the
-   builtin OOPs into _gst_nil_oop et al.  */
+   builtin OOPs into _gst_nil_oop et al. */
 static void alloc_oop_table (size_t);
 
 /* Free N slots from the beginning of the queue Q and return a pointer
@@ -124,7 +124,7 @@ static OOP *queue_get (surv_space *q, int n);
    to their base.  */
 static OOP *queue_put (surv_space *q, OOP *src, int n);
 
-/* Move an object from survivor space to oldspace.  */
+/* Move an object from survivor space to oldspace. */
 static void tenure_one_object ();
 
 /* Initialize an allocation heap with the oldspace hooks set.  */
@@ -150,7 +150,7 @@ static void finished_incremental_gc ();
 /* Gather statistics.  */
 static void update_stats (unsigned long *last, double *between, double *duration);
 
-/* The copying collector.  */
+/* The copying collector. */
 static inline void copy_oops (void);
 
 /* Grey ranges are generated in two cases.  The standard one is when we
@@ -185,20 +185,18 @@ static void oldspace_before_freeing (heap_data *h, heap_block *blk, size_t sz);
 static int oldspace_sigsegv_handler (void* fault_address, int serious);
 #endif
 
-/* Hook that triggers garbage collection.  */
+/* Hook that triggers garbage collection. */
 static void oldspace_nomemory (heap_data *h, size_t sz);
 
 /* Answer the number of fields to be scanned in the object starting
    at OBJ, with the given FLAGS on its OOP.  */
 static int scanned_fields_in (mst_Object obj, int flags) ATTRIBUTE_PURE;
 
-/* The mark phase of oldspace GC.  */
+/* The mark phase of oldspace GC. */
 static inline void mark_oops (void);
 
-/* Communicate to the finalization thread which objects have to be sent
-   the #mourn message.
-
-   When one of the objects pointed to by a weak object have no other
+/* Performs special checks on weak and ephemeron objects.  When one
+   of the objects pointed to by a weak object have no other
    references, the slot of the weak object is replaced by a zero and
    the #mourn message is sent to it.  Ephemerons' keys are checked for
    reachability after non-ephemerons are marked, and if no objects outside
@@ -272,7 +270,7 @@ _gst_init_mem (size_t eden, size_t survivor, size_t old,
       if (survivor)
         _gst_tenure_all_survivors ();
 
-      if (old && old != _gst_mem.old->heap_total)
+      if (old != _gst_mem.old->heap_total)
         _gst_grow_memory_to (old);
     }
 
@@ -296,9 +294,6 @@ _gst_init_mem (size_t eden, size_t survivor, size_t old,
   if (big_object_threshold)
     _gst_mem.big_object_threshold = big_object_threshold;
 
-  if (_gst_mem.eden.totalSize < _gst_mem.big_object_threshold)
-    _gst_mem.big_object_threshold = _gst_mem.eden.totalSize;
-
   if (grow_threshold_percent)
     _gst_mem.grow_threshold_percent = grow_threshold_percent;
 
@@ -337,7 +332,7 @@ void _gst_update_object_memory_oop (OOP oop)
   for (;;) {
     numScavenges = _gst_mem.numScavenges;
     data = (gst_object_memory) OOP_TO_OBJ (oop);
-    data->bytesPerOOP = FROM_INT (sizeof (PTR));
+    data->bytesPerOOP = FROM_INT (SIZEOF_CHAR_P);
     data->bytesPerOTE = FROM_INT (sizeof (struct OOP) +
 				  sizeof (gst_object_header));
 
@@ -422,7 +417,7 @@ _gst_init_oop_table (size_t size)
     {
       _gst_char_object_table[i].objSize =
 	FROM_INT (ROUNDED_WORDS (sizeof (struct gst_character)));
-      _gst_char_object_table[i].charVal = FROM_INT (i);
+      _gst_char_object_table[i].charVal = (char) i;
       _gst_mem.ot[i + CHAR_OBJECT_BASE].object =
 	(mst_Object) & _gst_char_object_table[i];
       _gst_mem.ot[i + CHAR_OBJECT_BASE].flags =
@@ -434,7 +429,7 @@ _gst_init_oop_table (size_t size)
 void
 alloc_oop_table (size_t size)
 {
-  size_t bytes;
+  long bytes;
   OOP oop;
 
   _gst_mem.ot_size = size;
@@ -445,9 +440,9 @@ alloc_oop_table (size_t size)
     nomemory (true);
 
   _gst_mem.ot = &_gst_mem.ot_base[-FIRST_OOP_INDEX];
-  _gst_nil_oop = &_gst_mem.ot[NIL_OOP_INDEX];
-  _gst_true_oop = &_gst_mem.ot[TRUE_OOP_INDEX];
-  _gst_false_oop = &_gst_mem.ot[FALSE_OOP_INDEX];
+  _gst_nil_oop = &_gst_mem.ot[nilOOPIndex];
+  _gst_true_oop = &_gst_mem.ot[trueOOPIndex];
+  _gst_false_oop = &_gst_mem.ot[falseOOPIndex];
 
   _gst_mem.num_free_oops = size;
 
@@ -464,7 +459,7 @@ alloc_oop_table (size_t size)
 mst_Boolean
 _gst_realloc_oop_table (size_t newSize)
 {
-  size_t bytes;
+  long bytes;
   OOP oop;
 
   bytes = (newSize - _gst_mem.ot_size) * sizeof (struct OOP);
@@ -474,7 +469,7 @@ _gst_realloc_oop_table (size_t newSize)
   if (!_gst_heap_sbrk (_gst_oop_heap, bytes))
     {
       /* try to recover.  Note that we cannot move the OOP table like
-         we do with the object data.  */
+         we do with the object data. */
       nomemory (false);
       return (false);
     }
@@ -518,6 +513,21 @@ _gst_init_builtin_objects_classes (void)
     _gst_char_object_table[i].objClass = _gst_char_class;
 }
 
+void
+_gst_fixup_metaclass_objects (void)
+{
+  OOP oop;
+  mst_Object object;
+
+  for (oop = _gst_mem.ot;
+       oop <= _gst_mem.last_allocated_oop; oop++)
+    {
+      object = OOP_TO_OBJ (oop);
+      if (IS_OOP_VALID (oop) && IS_NIL (object->objClass))
+	object->objClass = _gst_metaclass_class;
+    }
+}
+
 OOP
 _gst_find_an_instance (OOP class_oop)
 {
@@ -605,25 +615,16 @@ _gst_swap_objects (OOP oop1,
   if (oop1->flags & F_WEAK)
     _gst_make_oop_non_weak (oop1);
 
+  tempOOP = *oop2;		/* note structure assignment going on here */
+  *oop2 = *oop1;
+  *oop1 = tempOOP;
+
   /* Put the two objects in the same generation.  FIXME: this can be
      a cause of early tenuring, especially since one of them is often
      garbage!  */
   if ((oop1->flags & F_OLD) ^ (oop2->flags & F_OLD))
     _gst_tenure_oop ((oop1->flags & F_OLD) ? oop2 : oop1);
 
-  tempOOP = *oop2;		/* note structure assignment going on here */
-  *oop2 = *oop1;
-  *oop1 = tempOOP;
-
-  /* If the incremental GC has reached oop1 but not oop2 (or vice versa),
-     this flag will end up in the wrong OOP, i.e. in the one that has already
-     been scanned by the incremental GC.  Restore things.  */
-  if ((oop1->flags & F_REACHABLE) ^ (oop2->flags & F_REACHABLE))
-    {
-      oop1->flags ^= F_REACHABLE;
-      oop2->flags ^= F_REACHABLE;
-    }
-
   if (oop2->flags & F_WEAK)
     _gst_make_oop_weak (oop2);
 
@@ -640,7 +641,7 @@ _gst_make_oop_fixed (OOP oop)
   if (oop->flags & F_FIXED)
     return;
 
-  size = SIZE_TO_BYTES (TO_INT(oop->object->objSize));
+  size = TO_INT(oop->object->objSize) * SIZEOF_LONG;
   newObj = (mst_Object) _gst_mem_alloc (_gst_mem.fixed, size);
   if (!newObj)
     abort ();
@@ -665,7 +666,7 @@ _gst_tenure_oop (OOP oop)
 
   if (!(oop->flags & F_FIXED))
     {
-      int size = SIZE_TO_BYTES (TO_INT(oop->object->objSize));
+      int size = TO_INT(oop->object->objSize) * SIZEOF_LONG;
       newObj = (mst_Object) _gst_mem_alloc (_gst_mem.old, size);
       if (!newObj)
         abort ();
@@ -693,10 +694,7 @@ _gst_alloc_obj (size_t size,
 
   /* We don't want to have allocPtr pointing to the wrong thing during
      GC, so we use a local var to hold its new value */
-  newAllocPtr = _gst_mem.eden.allocPtr + BYTES_TO_SIZE (size);
-
-  if UNCOMMON (size >= _gst_mem.big_object_threshold)
-    return _gst_alloc_old_obj (size, p_oop);
+  newAllocPtr = _gst_mem.eden.allocPtr + (size >> LONG_SHIFT);
 
   if UNCOMMON (newAllocPtr >= _gst_mem.eden.maxPtr)
     {
@@ -704,10 +702,13 @@ _gst_alloc_obj (size_t size,
       newAllocPtr = _gst_mem.eden.allocPtr + size;
     }
 
+  if UNCOMMON (size >= _gst_mem.big_object_threshold)
+    return _gst_alloc_old_obj (size, p_oop);
+
   p_instance = (mst_Object) _gst_mem.eden.allocPtr;
   _gst_mem.eden.allocPtr = newAllocPtr;
   *p_oop = alloc_oop (p_instance, _gst_mem.active_flag);
-  p_instance->objSize = FROM_INT (BYTES_TO_SIZE (size));
+  p_instance->objSize = FROM_INT (size >> LONG_SHIFT);
 
   return p_instance;
 }
@@ -720,7 +721,7 @@ _gst_alloc_old_obj (size_t size,
   mst_Object p_instance;
 
   size = ROUNDED_BYTES (size);
-  newAllocPtr = _gst_mem.eden.allocPtr + BYTES_TO_SIZE (size);
+  newAllocPtr = _gst_mem.eden.allocPtr + (size >> LONG_SHIFT);
 
   /* If the object is big enough, we put it directly in oldspace.  */
   p_instance = (mst_Object) _gst_mem_alloc (_gst_mem.old, size);
@@ -743,7 +744,7 @@ _gst_alloc_old_obj (size_t size,
 
 ok:
   *p_oop = alloc_oop (p_instance, F_OLD);
-  p_instance->objSize = FROM_INT (BYTES_TO_SIZE (size));
+  p_instance->objSize = FROM_INT (size >> LONG_SHIFT);
   return p_instance;
 }
 
@@ -786,7 +787,7 @@ oldspace_after_allocating (heap_data *h, heap_block *blk, size_t sz)
   printf ("Allocating oldspace page at %p (%d)\n", blk, sz);
 #endif
 
-  add_to_grey_list ((OOP *) blk, sz / sizeof (PTR), NULL);
+  add_to_grey_list ((OOP *) blk, sz / SIZEOF_CHAR_P, NULL);
   _gst_mem.rememberedTableEntries++;
 }
 
@@ -846,7 +847,7 @@ int oldspace_sigsegv_handler (void* fault_address, int serious)
      reentering = 1;
    }
 
-  page = (char *) fault_address - ((intptr_t) fault_address & (getpagesize() - 1));
+  page = (char *) fault_address - ((long) fault_address & (getpagesize() - 1));
   errno = 0;
   if (_gst_mem_protect (page, getpagesize(), PROT_READ | PROT_WRITE) == -1 &&
       (errno == ENOMEM || errno == EFAULT))
@@ -867,7 +868,7 @@ int oldspace_sigsegv_handler (void* fault_address, int serious)
 #endif
 
   _gst_mem.rememberedTableEntries++;
-  add_to_grey_list ((PTR) page, getpagesize() / sizeof (PTR), NULL);
+  add_to_grey_list ((PTR) page, getpagesize() / SIZEOF_CHAR_P, NULL);
   return !reentered;
 }
 #endif
@@ -944,7 +945,7 @@ _gst_compact (size_t new_heap_limit)
     if ((oop->flags & (F_OLD | F_FIXED)) == F_OLD)
       {
         mst_Object new;
-        size_t size = SIZE_TO_BYTES (TO_INT (oop->object->objSize));
+        size_t size = SIZEOF_LONG * TO_INT (oop->object->objSize);
         new = _gst_mem_alloc (new_heap, size);
         memcpy (new, oop->object, size);
         _gst_mem_free (_gst_mem.old, oop->object);
@@ -972,7 +973,7 @@ _gst_global_compact ()
 void
 _gst_global_gc (int next_allocation)
 {
-  const char *s;
+  char *s;
   int old_limit;
 
   _gst_mem.numGlobalGCs++;
@@ -987,7 +988,7 @@ _gst_global_gc (int next_allocation)
          this scanning is apparent.  Note the use of stderr for the
          printed message.  The idea here was that generated output
          could be treated as Smalltalk code, HTML or whatever else you
-         want without harm.  */
+         want without harm. */
       fflush (stdout);
       fprintf (stderr, "\"Global garbage collection... ");
       fflush (stderr);
@@ -1081,7 +1082,7 @@ _gst_scavenge (void)
 	 this scanning is apparent.  Note the use of stderr for the
 	 printed message.  The idea here was that generated output
 	 could be treated as Smalltalk code, HTML or whatever else you
-	 want without harm.  */
+	 want without harm. */
       fflush (stdout);
       fprintf (stderr, "\"Scavenging... ");
       fflush (stderr);
@@ -1147,7 +1148,7 @@ _gst_finish_incremental_gc ()
 
   for (oop = _gst_mem.highest_swept_oop, firstOOP = _gst_mem.last_swept_oop;
        --oop > firstOOP; oop->flags &= ~F_REACHABLE)
-    if (!IS_OOP_VALID_GC (oop))
+    if (!IS_OOP_VALID(oop))
       {
         _gst_sweep_oop (oop);
 	_gst_mem.num_free_oops++;
@@ -1169,10 +1170,10 @@ finished_incremental_gc (void)
     _gst_mem.factor * stats.reclaimedOldSpaceBytesSinceLastGlobalGC +
     (1 - _gst_mem.factor) * _gst_mem.reclaimedBytesPerScavenge;
 
-#ifdef ENABLE_JIT_TRANSLATION
+#ifdef USE_JIT_TRANSLATION
   /* Go and really free the blocks associated to garbage collected
-     native code.  */
-  _gst_free_released_native_code ();
+     native code. */
+  free_released_native_code ();
 #endif
 }
 
@@ -1193,7 +1194,7 @@ _gst_incremental_gc_step ()
 	  break;
 	}
 
-      if (!IS_OOP_VALID_GC (oop))
+      if (!IS_OOP_VALID(oop))
         {
           i++;
           _gst_sweep_oop (oop);
@@ -1212,20 +1213,20 @@ reset_incremental_gc (OOP firstOOP)
 
   /* This loop is the same as that in alloc_oop.  Skip low OOPs
      that are allocated */
-  for (oop = firstOOP; IS_OOP_VALID_GC (oop); oop->flags &= ~F_REACHABLE, oop++)
-#if defined(ENABLE_JIT_TRANSLATION)
+  for (oop = firstOOP; IS_OOP_VALID (oop); oop->flags &= ~F_REACHABLE, oop++)
+#if defined(USE_JIT_TRANSLATION)
     if (oop->flags & F_XLAT)
       {
         if (oop->flags & F_XLAT_REACHABLE)
           /* Reachable, and referenced by active contexts.  Keep it
-             around.  */
+             around. */
           oop->flags &= ~F_XLAT_2NDCHANCE;
         else
           {
             /* Reachable, but not referenced by active contexts.  We
-               give it a second chance...  */
+               give it a second chance... */
             if (oop->flags & F_XLAT_2NDCHANCE)
-              _gst_release_native_code (oop);
+              release_native_code (oop);
 
             oop->flags ^= F_XLAT_2NDCHANCE;
           }
@@ -1241,7 +1242,7 @@ reset_incremental_gc (OOP firstOOP)
   _gst_finish_incremental_gc ();
 #else
   /* Skip high OOPs that are unallocated.  */
-  for (oop = _gst_mem.last_allocated_oop; !IS_OOP_VALID_GC (oop); oop--)
+  for (oop = _gst_mem.last_allocated_oop; !IS_OOP_VALID (oop); oop--)
     _gst_sweep_oop (oop);
 
   _gst_mem.last_allocated_oop = oop;
@@ -1275,7 +1276,7 @@ _gst_sweep_oop (OOP oop)
   if (IS_OOP_FREE (oop))
     return;
 
-#ifdef ENABLE_JIT_TRANSLATION
+#ifdef USE_JIT_TRANSLATION
   if (oop->flags & F_XLAT)
     /* Unreachable, always free the native code.  Note
        it is *not* optional to free the code in this case -- and
@@ -1283,25 +1284,25 @@ _gst_sweep_oop (OOP oop)
        method could use the same OOP as this one and the
        old method one would be executed instead of the new
        one! */
-    _gst_release_native_code (oop);
+    release_native_code (oop);
 #endif
 
   if UNCOMMON (oop->flags & F_WEAK)
     _gst_make_oop_non_weak (oop);
 
-  /* Free unreachable oldspace objects.  */
+  /* Free unreachable oldspace objects. */
   if UNCOMMON (oop->flags & F_FIXED)
     {
       _gst_mem.numOldOOPs--;
       stats.reclaimedOldSpaceBytesSinceLastGlobalGC +=
-	SIZE_TO_BYTES (TO_INT (OOP_TO_OBJ (oop)->objSize));
+	SIZEOF_LONG * TO_INT (OOP_TO_OBJ (oop)->objSize);
       _gst_mem_free (_gst_mem.fixed, oop->object);
     }
   else if UNCOMMON (oop->flags & F_OLD)
     {
       _gst_mem.numOldOOPs--;
       stats.reclaimedOldSpaceBytesSinceLastGlobalGC +=
-	SIZE_TO_BYTES (TO_INT (OOP_TO_OBJ (oop)->objSize));
+	SIZEOF_LONG * TO_INT (OOP_TO_OBJ (oop)->objSize);
       _gst_mem_free (_gst_mem.old, oop->object);
     }
 
@@ -1311,37 +1312,29 @@ _gst_sweep_oop (OOP oop)
 void
 mourn_objects (void)
 {
-  mst_Object array;
+  OOP *pOOP;
   long size;
-  gst_processor_scheduler processor;
 
-  size = _gst_buffer_size () / sizeof (OOP);
-  if (!size)
-    return;
+  /* Make a local copy of the buffer */
+  size = _gst_buffer_size ();
+  pOOP = alloca (size);
+  _gst_copy_buffer (pOOP);
+  size /= SIZEOF_CHAR_P;
 
-  processor = (gst_processor_scheduler) OOP_TO_OBJ (_gst_processor_oop);
-  if (!IS_NIL (processor->gcArray))
-    {
-      _gst_errorf ("Too many garbage collections, finalizers missed!");
-      _gst_errorf ("This is a bug, please report.");
-    }
-  else
-    {
-      /* Copy the buffer into an Array */
-      array = new_instance_with (_gst_array_class, size, &processor->gcArray);
-      _gst_copy_buffer (array->data);
-      _gst_async_signal (processor->gcSemaphore);
-    }
+  while (size--)
+    _gst_msg_send (*pOOP++, _gst_mourn_symbol, NULL);
 }
 
 
 #define IS_QUEUE_SPLIT(q) ((q)->topPtr != (q)->allocPtr)
 
+#define QUEUE_NEXT(q, addr) (IS_QUEUE_SPLIT (q) && (addr) >= (q)->topPtr ? (q)->minPtr : (addr))
+
 OOP *
 queue_get (surv_space *q, int n)
 {
   OOP *result = q->tenurePtr;
-  q->filled -= n * sizeof (PTR);
+  q->filled -= n * SIZEOF_CHAR_P;
   q->tenurePtr += n;
 
   /* Check if the read pointer has to wrap.  */
@@ -1396,13 +1389,13 @@ queue_put (surv_space *q, OOP *src, int n)
     /* We are still extending towards the top.  Push further the
        valid area (which is space...topPtr and minPtr...allocPtr
        if topPtr != allocPtr (not circular yet), space...allocPtr
-       if topPtr == allocPtr (circular).  */
+       if topPtr == allocPtr (circular). */
     q->topPtr = newAlloc;
 
-  q->filled += n * sizeof (PTR);
-  q->allocated += n * sizeof (PTR);
+  q->filled += n * SIZEOF_CHAR_P;
+  q->allocated += n * SIZEOF_CHAR_P;
   q->allocPtr = newAlloc;
-  memcpy (result, src, n * sizeof (PTR));
+  memcpy (result, src, n * SIZEOF_CHAR_P);
   return result;
 }
 
@@ -1427,6 +1420,8 @@ tenure_one_object ()
       _gst_mem.scan.at = (OOP *) oop->object;
     }
 
+  /* Ephemerons are treated specially.  Since _gst_copy_an_oop itself
+     puts them in the tree of grey areas, we don't need to grey them.  */
   else if (_gst_mem.scan.queue_at == _gst_mem.tenuring_queue.tenurePtr)
     {
       mst_Object obj;
@@ -1436,14 +1431,8 @@ tenure_one_object ()
       printf ("Tenured OOP %p had not been scanned yet\n", oop);
 #endif
 
-      /* Since tenurePtr is going to advance by a place, we must
-	 keep the Cheney scan pointer up to date.  Check if it has
-	 to wrap!  */
-      _gst_mem.scan.queue_at++;
-      if (_gst_mem.scan.queue_at >= _gst_mem.tenuring_queue.topPtr
-          && IS_QUEUE_SPLIT (&_gst_mem.tenuring_queue))
-        _gst_mem.scan.queue_at = _gst_mem.tenuring_queue.minPtr;
-
+      _gst_mem.scan.queue_at = QUEUE_NEXT (&_gst_mem.tenuring_queue,
+					   _gst_mem.scan.queue_at + 1);
       _gst_tenure_oop (oop);
       obj = OOP_TO_OBJ (oop);
       numFields = scanned_fields_in (obj, oop->flags);
@@ -1463,7 +1452,7 @@ _gst_grey_oop_range (PTR from, size_t size)
   volatile char *last, *page;
 
   for (last = ((char *)from) + size,
-       page = ((char *)from) - ((intptr_t) from & (getpagesize() - 1));
+       page = ((char *)from) - ((long) from & (getpagesize() - 1));
        page < last;
        page += getpagesize())
     *page = *page;
@@ -1526,7 +1515,7 @@ check_weak_refs ()
           if (IS_INT (oop))
 	    continue;
 
-          if (!IS_OOP_VALID_GC (oop))
+          if (!IS_OOP_VALID (oop))
             {
               mourn = true;
 	      *field = _gst_nil_oop;
@@ -1628,7 +1617,7 @@ scan_grey_pages ()
 	  printf ("Protecting %p\n", node->base);
 #endif
 	  _gst_mem.rememberedTableEntries--;
-	  _gst_mem_protect ((PTR) node->base, node->n * sizeof(OOP), PROT_READ);
+	  _gst_mem_protect ((PTR) node->base, getpagesize(), PROT_READ);
 	  *next = node->next;
 	  xfree (node);
 	}
@@ -1727,7 +1716,7 @@ scanned_fields_in (mst_Object object,
   if COMMON (flags & F_CONTEXT)
     {
       gst_method_context ctx;
-      intptr_t methodSP;
+      long methodSP;
       ctx = (gst_method_context) object;
       methodSP = TO_INT (ctx->spOffset);
       return ctx->contextStack + methodSP + 1 - &ctx->objClass;
@@ -1737,7 +1726,7 @@ scanned_fields_in (mst_Object object,
       /* In general, there will be many instances of a class,
 	 but only the first time will it need to be copied;
          moreover, classes are often old.  So I'm
-	 marking this as uncommon.  */
+	 marking this as uncommon. */
       return UNCOMMON (!IS_OOP_COPIED (objClass));
     }
 }
@@ -1752,27 +1741,17 @@ cheney_scan (void)
   while (_gst_mem.scan.queue_at !=
 	   _gst_mem.tenuring_queue.allocPtr)
     {
-      OOP oop;
+      OOP oop = *_gst_mem.scan.queue_at;
       int i, numFields;
 
-      if (_gst_mem.scan.queue_at >= _gst_mem.tenuring_queue.topPtr)
-        _gst_mem.scan.queue_at = _gst_mem.tenuring_queue.minPtr;
-
-      if (_gst_mem.scan.queue_at == _gst_mem.tenuring_queue.allocPtr)
-	break;
-
-      oop = *_gst_mem.scan.queue_at;
-
 #if defined(GC_DEBUGGING)
-      if (!IS_OOP_ADDR (oop))
-	abort();
-
       printf (">Scan ");
       _gst_display_oop (oop);
 #endif
 
       _gst_mem.scan.current = oop;
-      _gst_mem.scan.queue_at++;
+      _gst_mem.scan.queue_at = QUEUE_NEXT (&_gst_mem.tenuring_queue,
+					   _gst_mem.scan.queue_at + 1);
 
       if (oop->flags & F_EPHEMERON)
 	continue;
@@ -1860,7 +1839,7 @@ _gst_copy_an_oop (OOP oop)
       /* Look for a child that has not been copied and move it
          near the object.  This improves the locality of reference.  
          We do not copy the class (that's the reason for the -1
-	 here).  */
+	 here). */
       n = scanned_fields_in (obj, oop->flags) - 1;
       if (oop->flags & F_EPHEMERON)
 	{
@@ -1904,10 +1883,10 @@ mark_ephemeron_oops (void)
   base = alloca (size);
   _gst_copy_buffer (base);
   _gst_reset_buffer ();
-  size /= sizeof (PTR);
+  size /= SIZEOF_CHAR_P;
 
   /* First pass: distinguish objects whose key was reachable from
-     the outside by clearing their F_EPHEMERON bit.  */
+     the outside by clearing their F_EPHEMERON bit. */
   for (pOOP = base, i = size; i--; pOOP++)
     {
       OOP oop = *pOOP;
@@ -1929,21 +1908,21 @@ mark_ephemeron_oops (void)
       int j;
 
       /* Find if the key is reachable from the objects (so that
-         we can mourn the ephemeron if this is not so).  */
+         we can mourn the ephemeron if this is not so). */
       key->flags &= ~F_REACHABLE;
 
       for (j = 1; j < num; j++)
         MAYBE_MARK_OOP (obj->data[j]);
 
       /* Remember that above we cleared F_EPHEMERON is the key
-         is alive.  */
+         is alive. */
       if (!IS_OOP_MARKED (key) && (oop->flags & F_EPHEMERON))
         _gst_add_buf_pointer (oop);
 
-      /* Ok, now mark the key.  */
+      /* Ok, now mark the key. */
       MAYBE_MARK_OOP (key);
 
-      /* Restore the flag in case it was cleared.  */
+      /* Restore the flag in case it was cleared. */
       oop->flags |= F_EPHEMERON;
     }
 }
@@ -2005,7 +1984,7 @@ _gst_mark_an_oop_internal (OOP oop,
 	  /* We reach this point if the object isn't to be marked.  The 
 	     code above contains a continue to tail recurse, so we
 	     cannot put the loop in a do...while and a goto is
-	     necessary here.  Speed is a requirement, so I'm doing it.  */
+	     necessary here.  Speed is a requirement, so I'm doing it. */
 	  if (curOOP < atEndOOP)
 	    goto iterationLoop;
 	}
@@ -2013,7 +1992,7 @@ _gst_mark_an_oop_internal (OOP oop,
 	{			/* just starting with this oop */
 	  OOP objClass;
 	  mst_Object object;
-	  uintptr_t size;
+	  unsigned long size;
 
 #if !defined (OPTIMIZE)
 	  if UNCOMMON (IS_OOP_FREE (oop))
@@ -2037,7 +2016,7 @@ _gst_mark_an_oop_internal (OOP oop,
 	  if UNCOMMON (oop->flags & F_CONTEXT)
 	    {
 	      gst_method_context ctx;
-	      intptr_t methodSP;
+	      long methodSP;
 	      ctx = (gst_method_context) object;
 	      methodSP = TO_INT (ctx->spOffset);
 	      /* printf("setting up for loop on context %x, sp = %d\n", 
@@ -2050,7 +2029,7 @@ _gst_mark_an_oop_internal (OOP oop,
 	    {
 	      /* In general, there will be many instances of a class,
 		 but only the first time will it be unmarked.  So I'm
-		 marking this as uncommon.  */
+		 marking this as uncommon. */
 	      if UNCOMMON (!IS_OOP_MARKED (objClass))
 		TAIL_MARK_OOP (objClass);
 
@@ -2069,7 +2048,7 @@ _gst_mark_an_oop_internal (OOP oop,
 	    }
 	}
       /* This point is reached if and only if nothing has to be marked
-         anymore in the current iteration. So exit.  */
+         anymore in the current iteration. So exit. */
       break;
     }				/* for(;;) */
 }
@@ -2093,8 +2072,8 @@ void
 _gst_inc_grow_registry (void)
 {
   OOP *oldBase;
-  unsigned oldPtrOffset;
-  unsigned oldRegistrySize, newRegistrySize;
+  unsigned long oldPtrOffset;
+  unsigned long oldRegistrySize, newRegistrySize;
 
   oldBase = _gst_mem.inc_base;
   oldPtrOffset = _gst_mem.inc_ptr - _gst_mem.inc_base;
