@@ -34,23 +34,8 @@
  ***********************************************************************/
 
 
-#include "gst.h"
 #include "gstpriv.h"
 
-#include <errno.h>
-#include <assert.h>
-#include <stdio.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <setjmp.h>
-#include "signalx.h"
-#include "socketx.h"
-
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
 #ifdef HAVE_SYS_TIMES_H
 # include <sys/times.h>
 #endif
@@ -59,10 +44,6 @@
 # include <sys/ioctl.h>
 #endif
 
-#ifdef HAVE_SYS_MMAN_H
-# include <sys/mman.h>
-#endif
-
 #ifdef HAVE_TERMIOS_H
 # include <termios.h>
 #endif
@@ -83,24 +64,6 @@
 #include <sys/timeb.h>
 #endif
 
-#ifdef HAVE_SYS_RESOURCE_H
-# include <sys/resource.h>
-#endif
-
-#if defined(HAVE_SYS_TIME_H) || defined(TIME_WITH_SYS_TIME)
-# include <sys/time.h>
-#endif
-#if !defined(HAVE_SYS_TIME_H) || defined(TIME_WITH_SYS_TIME)
-# include <time.h>
-#endif
-
-#ifdef STDC_HEADERS
-# include <string.h>
-# include <stdlib.h>
-#endif /* STDC_HEADERS */
-
-#include <stdarg.h>
-
 #ifdef WIN32
 # define WIN32_LEAN_AND_MEAN /* avoid including junk */
 # include <windows.h>
@@ -125,18 +88,16 @@
 #define O_BINARY     0
 #endif
 
-#ifdef __MSVCRT__
+#ifdef HAVE_SPAWNL
+#include <process.h>
 #ifndef P_WAIT
 #define P_WAIT       0
 #define P_NOWAIT     1
 #define P_OVERLAY    2
 #define P_NOWAITO    3
 #define P_DETACH     4
-
-#define WAIT_CHILD    0
-#define WAIT_GRANDCHILD       1
-#endif
-#endif
+#endif /* !P_WAIT */
+#endif /* HAVE_SPAWNL */
 
 #if defined(FASYNC) && !defined (O_ASYNC)
 #define O_ASYNC FASYNC
@@ -1000,8 +961,8 @@ open_pty (pty_info *pty)
 	break;
       }
 
-  pty->master = _open_osfhandle (hMaster, pty->access | O_BINARY);
-  return pty->master;
+    pty->master = _open_osfhandle (hMaster, pty->access | O_BINARY);
+    return pty->master;
   }
 #else
   return -1;
@@ -1013,7 +974,7 @@ init_pty (pty_info *pty)
 {
   int slave;
 
-#ifdef HAVE_FORK
+#ifndef HAVE_SPAWNL
   close (pty->master);
 #endif
 
@@ -1033,7 +994,7 @@ init_pty (pty_info *pty)
       exit (255);
     }
 
-#ifdef HAVE_SETSID
+#if !defined HAVE_SPAWNL && defined HAVE_SETSID
   if (setsid () < 0)
     {
       perror ("setsid");
@@ -1043,7 +1004,7 @@ init_pty (pty_info *pty)
 
   ioctl (slave, TIOCSCTTY, 0);
 #else
-#ifdef HAVE_SETSID
+#if !defined HAVE_SPAWNL && defined HAVE_SETSID
   if (setsid () < 0)
     {
       perror ("setsid");
@@ -1159,8 +1120,8 @@ _gst_open_pipe (const char *command,
 #ifdef HAVE_SPAWNL
   init_pty (&pty);
 
-#ifdef __unix__
-  result = spawnl (P_NOWAIT, "/bin/sh", "-c", command, NULL);
+#ifdef __CYGWIN__
+  result = spawnl (P_NOWAIT, "/bin/sh", "/bin/sh", "-c", command, NULL);
 #else
   result = spawnl (P_NOWAIT, getenv("COMSPEC"), "/C", command, NULL);
 #endif
@@ -1183,7 +1144,7 @@ _gst_open_pipe (const char *command,
       close (pty.save_stderr);
     }
 
-#else /* !HAVE_SPAWNV */
+#else /* !HAVE_SPAWNL */
   /* We suppose it is a system that has fork.  */
   result = fork ();
   if (result == 0)
@@ -1195,7 +1156,7 @@ _gst_open_pipe (const char *command,
       /*NOTREACHED*/
     }
 
-#endif /* !HAVE_SPAWNV */
+#endif /* !HAVE_SPAWNL */
 
   _gst_set_signal_handler (SIGPIPE, SIG_IGN);
 
@@ -1466,8 +1427,12 @@ _gst_debug (void)
 }
 
 /* We have three possible implementations: Win32, mmap-ing /dev/zero
-   or using anonymous mmaps, and anonymous mmaps with MAP_NORESERVE.  */
+   or using anonymous mmaps, and anonymous mmaps with MAP_NORESERVE. 
+   There is support for providing fallback implementations, but it
+   is not used currently.  */
+
 typedef struct heap_implementation {
+  mst_Boolean (*check) ();
   PTR (*reserve) (size_t);
   void (*release) (PTR, size_t);
   PTR (*commit) (PTR, size_t);
@@ -1480,47 +1445,64 @@ static void win32_release (PTR, size_t);
 static PTR win32_commit (PTR, size_t);
 static void win32_decommit (PTR, size_t);
 struct heap_implementation heap_impl_tab[] = {
-  { win32_reserve, win32_release, win32_commit, win32_decommit },
-  { NULL, NULL, NULL, NULL }
+  { NULL, win32_reserve, win32_release, win32_commit, win32_decommit },
+  { NULL, NULL, NULL, NULL, NULL }
 };
 #else /* !WIN32 */
-# if defined (MAP_ANON) && defined (MAP_NORESERVE)
+
+# ifdef MAP_NORESERVE
 static PTR noreserve_reserve (size_t);
-static PTR noreserve_commit (PTR, size_t);
 static void noreserve_decommit (PTR, size_t);
 #endif
+static mst_Boolean anon_mmap_check (void);
 static PTR anon_mmap_reserve (size_t);
 static void anon_mmap_release (PTR, size_t);
 static PTR anon_mmap_commit (PTR, size_t);
 
 struct heap_implementation heap_impl_tab[] = {
-# if defined (MAP_ANON) && defined (MAP_NORESERVE)
-  { noreserve_reserve, _gst_osmem_free, noreserve_commit, noreserve_decommit },
+# ifdef MAP_NORESERVE
+  { NULL, noreserve_reserve, _gst_osmem_free, anon_mmap_commit, noreserve_decommit },
 # endif
-  { anon_mmap_reserve, anon_mmap_release, anon_mmap_commit, _gst_osmem_free },
-  { NULL, NULL, NULL, NULL }
+  { anon_mmap_check, anon_mmap_reserve, anon_mmap_release, anon_mmap_commit, _gst_osmem_free },
+  { NULL, NULL, NULL, NULL, NULL }
 };
+
+# ifndef MAP_ANON
+static int dev_zero = -1;
+#  define anon_mmap(addr, size, prot, flags) \
+     mmap ((addr), (size), (prot), (flags), \
+	   (UNCOMMON(dev_zero == -1) \
+	    ? dev_zero = open ("/dev/zero", O_RDWR) \
+	    : dev_zero), 0)
+# else
+#  define anon_mmap(addr, size, prot, flags) \
+     mmap ((addr), (size), (prot), MAP_ANON | (flags), -1, 0)
+# endif
 #endif /* !WIN32 */
 
 
 
-static heap_implementation *impl = heap_impl_tab;
-
-#if !defined (WIN32) && !defined (MAP_ANON)
-static int fd = -1;
-#endif
+static heap_implementation *impl;
 
 PTR
 _gst_osmem_reserve (size_t size)
 {
-  PTR base;
+  if (!impl)
+    {
+      /* Find an implementation of heaps that is ok for this machine.
+         The check is done at run-time because it is cheap.  */
+      for (impl = heap_impl_tab; impl->reserve; impl++)
+        if (!impl->check || impl->check ())
+	  return impl->reserve (size);
 
-  for (; impl->reserve; impl++)
-    if ((base = impl->reserve (size)))
-      return base;
-
-  errno = ENOMEM;
-  return (NULL);
+      /* Not found, check again the next time just in case and return
+         ENOMEM.  */
+      impl = NULL;
+      errno = ENOMEM;
+      return (NULL);
+    }
+  else
+    return impl->reserve (size);
 }
 
 void
@@ -1553,24 +1535,17 @@ _gst_osmem_alloc (size_t size)
       if (result)
         return result;
 
-      VirtualFree (addr, size, MEM_DECOMMIT);
+      VirtualFree (addr, size, MEM_RELEASE);
     }
 
+  errno = ENOMEM;
   return NULL;
 #else
-#ifndef MAP_ANON
-  /* Open the file descriptor we need for anonymous mmaps */
-  if (fd == -1)
-    fd = open ("/dev/zero", O_RDWR);
 
-  addr = mmap (NULL, size,
-        PROT_READ | PROT_WRITE | PROT_EXEC,
-        MAP_PRIVATE, fd, 0);
-#else
-  addr = mmap (NULL, size,
-        PROT_READ | PROT_WRITE | PROT_EXEC,
-        MAP_ANON | MAP_PRIVATE, -1, 0);
-#endif
+  addr = anon_mmap (NULL, size,
+		    PROT_READ | PROT_WRITE | PROT_EXEC,
+		    MAP_PRIVATE);
+
   return addr == MAP_FAILED ? NULL : addr;
 #endif
 }
@@ -1594,7 +1569,12 @@ _gst_osmem_free (PTR ptr, size_t size)
 PTR
 win32_reserve (size_t size)
 {
-  return VirtualAlloc(NULL, size, MEM_RESERVE, PAGE_NOACCESS);
+  PTR base;
+  base = VirtualAlloc(NULL, size, MEM_RESERVE, PAGE_NOACCESS);
+  if (!base)
+    errno = ENOMEM;
+
+  return base;
 }
 
 void
@@ -1606,7 +1586,12 @@ win32_release (PTR base, size_t size)
 PTR
 win32_commit (PTR base, size_t size)
 {
-  return VirtualAlloc (base, size, MEM_COMMIT, PAGE_EXECUTE_READWRITE);
+  PTR addr;
+  addr = VirtualAlloc (base, size, MEM_COMMIT, PAGE_EXECUTE_READWRITE);
+  if (!addr)
+    errno = ENOMEM;
+
+  return addr;
 }
 
 void
@@ -1617,27 +1602,15 @@ win32_decommit (PTR base, size_t size)
 
 #else /* !WIN32 */
 
-#if defined (MAP_ANON) && defined (MAP_NORESERVE)
+#ifdef MAP_NORESERVE
 /* Implementation of the four basic primitives when MAP_NORESERVE
    is available.  */
 
 PTR
 noreserve_reserve (size_t size)
 {
-  PTR result = mmap (NULL, size, PROT_NONE,
-		     MAP_ANON | MAP_PRIVATE | MAP_NORESERVE,
-		     -1, 0);
-
-  return result == MAP_FAILED ? NULL : result;
-}
-
-PTR
-noreserve_commit (PTR base, size_t size)
-{
-  PTR result = mmap (base, size,
-		     PROT_READ | PROT_WRITE | PROT_EXEC,
-		     MAP_ANON | MAP_PRIVATE | MAP_FIXED,
-		     -1, 0);
+  PTR result = anon_mmap (NULL, size, PROT_NONE,
+			  MAP_PRIVATE | MAP_NORESERVE);
 
   return result == MAP_FAILED ? NULL : result;
 }
@@ -1645,36 +1618,34 @@ noreserve_commit (PTR base, size_t size)
 void
 noreserve_decommit (PTR base, size_t size)
 {
-  mmap (base, size,
-	PROT_NONE, MAP_ANON | MAP_PRIVATE | MAP_NORESERVE | MAP_FIXED,
-	-1, 0);
+  anon_mmap (base, size, PROT_NONE,
+	     MAP_PRIVATE | MAP_NORESERVE | MAP_FIXED);
 }
 #endif
 
 /* Implementation of the four basic primitives when MAP_NORESERVE
-   is not available.  */
+   is not available.  Not foolproof, but we try to keep the balance
+   between simplicity and usefulness.  */
 
 static char *baseaddr;
-static PTR find_heap_base (void);
 
 PTR
 anon_mmap_reserve (size_t size)
 {
   PTR base;
 
-  if (!baseaddr)
+  /* We must check for overflows in baseaddr!  */
+  if (((unsigned long) baseaddr) + size < (unsigned long) baseaddr)
     {
-#ifndef MAP_ANON
-      /* Open the file descriptor we need for anonymous mmaps */
-      if (fd == -1)
-        fd = open ("/dev/zero", O_RDWR);
-#endif
-      baseaddr = find_heap_base ();
+      errno = ENOMEM;
+      return NULL;
+    }
+  else
+    {
+      base = baseaddr;
+      baseaddr += size;
+      return base;
     }
-
-  base = baseaddr;
-  baseaddr += size;
-  return base;
 }
 
 void
@@ -1687,17 +1658,9 @@ anon_mmap_release (PTR base, size_t size)
 PTR
 anon_mmap_commit (PTR base, size_t size)
 {
-#ifndef MAP_ANON
-  PTR result = mmap (base, size,
-		     PROT_READ | PROT_WRITE | PROT_EXEC,
-		     MAP_PRIVATE | (base ? MAP_FIXED : 0),
-		     fd, 0);
-#else
-  PTR result = mmap (base, size,
-		     PROT_READ | PROT_WRITE | PROT_EXEC,
-		     MAP_ANON | MAP_PRIVATE | (base ? MAP_FIXED : 0),
-		     -1, 0);
-#endif
+  PTR result = anon_mmap (base, size,
+		          PROT_READ | PROT_WRITE | PROT_EXEC,
+		          MAP_PRIVATE | MAP_FIXED);
 
   return result;
 }
@@ -1725,8 +1688,8 @@ not_mapped (sig)
   longjmp (already_mapped, 1);
 }
 
-PTR
-find_heap_base (void)
+mst_Boolean
+anon_mmap_check (void)
 {
   volatile char *mmapGuess, *higher, *lower;	/* reference addresses */
   volatile char *first = NULL, *second;	/* probed addresses */
@@ -1749,6 +1712,9 @@ find_heap_base (void)
   volatile SigHandler oldSegvHandler;
   int pagesize = getpagesize();
 
+  if (baseaddr)
+    return (true);
+
   /* Placate GNU C's warnings about clobbered variables */
   testPtr = (volatile int *) &higher;
   testPtr = (volatile int *) &lower;
@@ -1760,7 +1726,7 @@ find_heap_base (void)
   oldSegvHandler = _gst_set_signal_handler (SIGSEGV, not_mapped);
 
   /* Get two reference addresses which we will base ourselves on */
-  mmapGuess = higher = anon_mmap_commit (NULL, pagesize);
+  mmapGuess = higher = _gst_osmem_alloc (pagesize);
   lower = sbrk (0);
 
   if (higher < lower)
@@ -1821,11 +1787,11 @@ find_heap_base (void)
   munmap ((char *) mmapGuess, pagesize);
 
   if (first == NULL || *step == -1)
-    {
-      _gst_errorf ("Could not find a place for the Smalltalk heaps.");
-      return (NULL);
-    }
+    return (false);
   else
-    return (PTR) (first);
+    {
+      baseaddr = (PTR) first;
+      return (true);
+    }
 }
 #endif /* !WIN32 */
