@@ -37,18 +37,6 @@
 #  include <dmalloc.h>
 #endif
 
-#ifdef _WIN32
-#  ifdef DLL_EXPORT
-#    define SNV_SCOPE	__declspec(dllexport)
-#  endif
-#  ifdef LIBSNPRINTFV_DLL_IMPORT
-#    define SNV_SCOPE	extern __declspec(dllimport)
-#  endif
-#endif
-#ifndef SNV_SCOPE
-#  define SNV_SCOPE	extern
-#endif
-
 /* This is the API we use throughout libsnprintfv. */
 #define snv_new(type, count)		\
 		((type*)snv_malloc((size_t)sizeof(type) * (count)))
@@ -73,7 +61,7 @@ extern "C" {
  * Return value:
  * The pointer to the newly-allocated memory area.
  */
-SNV_SCOPE snv_pointer (*snv_malloc) PARAMS((size_t count));
+SNV_SCOPE snv_pointer (*snv_malloc) (size_t count);
 
 /**
  * snv_realloc:
@@ -87,7 +75,7 @@ SNV_SCOPE snv_pointer (*snv_malloc) PARAMS((size_t count));
  * The pointer to the newly-allocated memory area, possibly
  * the same as @old.
  */
-SNV_SCOPE snv_pointer (*snv_realloc) PARAMS((snv_pointer old, size_t count));
+SNV_SCOPE snv_pointer (*snv_realloc) (snv_pointer old, size_t count);
 
 /**
  * snv_free:
@@ -95,12 +83,12 @@ SNV_SCOPE snv_pointer (*snv_realloc) PARAMS((snv_pointer old, size_t count));
  *
  * Frees a block of memory pointed to by @old.
  */
-SNV_SCOPE void (*snv_free) PARAMS((snv_pointer old));
+SNV_SCOPE void (*snv_free) (snv_pointer old);
 
 /* And these are reimplemented tout court because they are
    not fully portable.  */
-extern snv_pointer snv_xrealloc PARAMS((snv_pointer old, size_t count));
-extern char* snv_strdup PARAMS((const char *str));
+extern snv_pointer snv_xrealloc (snv_pointer old, size_t count);
+extern char* snv_strdup (const char *str);
 
 #ifdef __cplusplus
 }
